@@ -1,18 +1,21 @@
+mod dependency_manager;
+
 use std::{
+    cell::RefCell,
     collections::{HashMap, HashSet},
     process::Command,
+    rc::Rc,
     time::Instant,
 };
 
 use camino::{Utf8Path, Utf8PathBuf};
 use ecow::{EcoString, eco_format};
 use flate2::read::GzDecoder;
-use futures::future;
 use gleam_core::{
     Error, Result,
     build::{Mode, Target, Telemetry},
     config::PackageConfig,
-    dependency,
+    dependency::{self, PackageFetchError},
     error::{FileIoAction, FileKind, ShellCommandFailureReason, StandardIoAction},
     hex::{self, HEXPM_PUBLIC_KEY},
     io::{HttpClient as _, TarUnpacker, WrappedReader},
@@ -25,6 +28,8 @@ use itertools::Itertools;
 use same_file::is_same_file;
 use strum::IntoEnumIterator;
 
+pub use dependency_manager::DependencyManagerConfig;
+
 #[cfg(test)]
 mod tests;
 
@@ -50,6 +55,14 @@ static UTF8_SYMBOLS: Symbols = Symbols {
     right: "─",
 };
 
+/// When set to `Yes`, the cli will check for major version updates of direct dependencies and
+/// print them to the console if the major versions are not upgradeable due to constraints.
+#[derive(Debug, Clone, Copy)]
+pub enum CheckMajorVersions {
+    Yes,
+    No,
+}
+
 pub fn list(paths: &ProjectPaths) -> Result<()> {
     let (_, manifest) = get_manifest_details(paths)?;
     list_manifest_packages(std::io::stdout(), manifest)
@@ -80,15 +93,18 @@ pub fn tree(paths: &ProjectPaths, options: TreeOptions) -> Result<()> {
 fn get_manifest_details(paths: &ProjectPaths) -> Result<(PackageConfig, Manifest)> {
     let runtime = tokio::runtime::Runtime::new().expect("Unable to start Tokio async runtime");
     let config = crate::config::root_config(paths)?;
-    let (_, manifest) = get_manifest(
-        paths,
+    let package_fetcher = PackageFetcher::new(runtime.handle().clone());
+    let dependency_manager = DependencyManagerConfig {
+        use_manifest: UseManifest::Yes,
+        check_major_versions: CheckMajorVersions::No,
+    }
+    .into_dependency_manager(
         runtime.handle().clone(),
+        package_fetcher,
+        cli::Reporter::new(),
         Mode::Dev,
-        &config,
-        &cli::Reporter::new(),
-        UseManifest::Yes,
-        Vec::new(),
-    )?;
+    );
+    let (_, manifest) = dependency_manager.get_manifest(paths, &config, Vec::new())?;
     Ok((config, manifest))
 }
 
@@ -96,7 +112,7 @@ fn list_manifest_packages<W: std::io::Write>(mut buffer: W, manifest: Manifest)
     manifest
         .packages
         .into_iter()
-        .try_for_each(|package| writeln!(buffer, "{} {}", package.name, package.version))
+        .try_for_each(|package| writeln!(buffer, "{}\t{}", package.name, package.version))
         .map_err(|e| Error::StandardIo {
             action: StandardIoAction::Write,
             err: Some(e.kind()),
@@ -131,7 +147,7 @@ fn list_package_and_dependencies_tree<W: std::io::Write>(
         );
 
         tree.iter()
-            .try_for_each(|line| writeln!(buffer, "{}", line))
+            .try_for_each(|line| writeln!(buffer, "{line}"))
             .map_err(|e| Error::StandardIo {
                 action: StandardIoAction::Write,
                 err: Some(e.kind()),
@@ -215,7 +231,10 @@ pub fn update(paths: &ProjectPaths, packages: Vec<String>) -> Result<()> {
         cli::Reporter::new(),
         None,
         packages.into_iter().map(EcoString::from).collect(),
-        use_manifest,
+        DependencyManagerConfig {
+            use_manifest,
+            check_major_versions: CheckMajorVersions::Yes,
+        },
     )?;
 
     Ok(())
@@ -295,7 +314,10 @@ fn remove_extra_requirements(config: &PackageConfig, manifest: &mut Manifest) ->
 pub fn parse_gleam_add_specifier(package: &str) -> Result<(EcoString, Requirement)> {
     let Some((package, version)) = package.split_once('@') else {
         // Default to the latest version available.
-        return Ok((package.into(), Requirement::hex(">= 0.0.0")));
+        return Ok((
+            package.into(),
+            Requirement::hex(">= 0.0.0").expect("'>= 0.0.0' should be a valid pubgrub range"),
+        ));
     };
 
     // Parse the major and minor from the provided semantic version.
@@ -338,7 +360,7 @@ pub fn parse_gleam_add_specifier(package: &str) -> Result<(EcoString, Requiremen
                 ),
             });
         }
-    };
+    }?;
 
     Ok((package.into(), requirement))
 }
@@ -348,77 +370,72 @@ pub fn download<Telem: Telemetry>(
     telemetry: Telem,
     new_package: Option<(Vec<(EcoString, Requirement)>, bool)>,
     packages_to_update: Vec<EcoString>,
-    // If true we read the manifest from disc. If not set then we ignore any
-    // manifest which will result in the latest versions of the dependency
-    // packages being resolved (not the locked ones).
-    use_manifest: UseManifest,
+    config: DependencyManagerConfig,
 ) -> Result<Manifest> {
-    let span = tracing::info_span!("download_deps");
-    let _enter = span.enter();
-
-    let mode = Mode::Dev;
-
-    // We do this before acquiring the build lock so that we don't create the
-    // build directory if there is no gleam.toml
-    crate::config::ensure_config_exists(paths)?;
-
-    let lock = BuildLock::new_packages(paths)?;
-    let _guard: Guard = lock.lock(&telemetry)?;
-
-    let fs = ProjectIO::boxed();
-
-    // Read the project config
-    let mut config = crate::root_config(paths)?;
-    let project_name = config.name.clone();
-
-    // Insert the new packages to add, if it exists
-    if let Some((packages, dev)) = new_package {
-        for (package, requirement) in packages {
-            if dev {
-                _ = config.dev_dependencies.insert(package, requirement);
-            } else {
-                _ = config.dependencies.insert(package, requirement);
-            };
-        }
-    }
-
     // Start event loop so we can run async functions to call the Hex API
     let runtime = tokio::runtime::Runtime::new().expect("Unable to start Tokio async runtime");
+    let package_fetcher = PackageFetcher::new(runtime.handle().clone());
 
-    // Determine what versions we need
-    let (manifest_updated, manifest) = get_manifest(
-        paths,
+    let dependency_manager = config.into_dependency_manager(
         runtime.handle().clone(),
-        mode,
-        &config,
-        &telemetry,
-        use_manifest,
-        packages_to_update,
-    )?;
-    let local = LocalPackages::read_from_disc(paths)?;
+        package_fetcher,
+        telemetry,
+        Mode::Dev,
+    );
 
-    // Remove any packages that are no longer required due to gleam.toml changes
-    remove_extra_packages(paths, &local, &manifest, &telemetry)?;
+    dependency_manager.download(paths, new_package, packages_to_update)
+}
 
-    // Download them from Hex to the local cache
-    runtime.block_on(add_missing_packages(
-        paths,
-        fs,
-        &manifest,
-        &local,
-        project_name,
-        &telemetry,
-    ))?;
+fn pretty_print_major_versions_available(versions: dependency::PackageVersionDiffs) -> String {
+    let total_lines = versions.len() + 3;
+    let versions = versions
+        .iter()
+        .map(|(name, (v1, v2))| (name, v1.to_string(), v2.to_string()))
+        .sorted();
 
-    if manifest_updated {
-        // Record new state of the packages directory
-        // TODO: test
-        tracing::debug!("writing_manifest_toml");
-        write_manifest_to_disc(paths, &manifest)?;
+    let longest_parts = versions.clone().fold(
+        (0, 0, 0),
+        |(max_name, max_curr, max_major), (name, curr, major)| {
+            (
+                max_name.max(name.len()),
+                max_curr.max(curr.len()),
+                max_major.max(major.len()),
+            )
+        },
+    );
+
+    let (longest_package_name_length, longest_current_version_length, longest_major_version_length) =
+        longest_parts;
+
+    let mut output_string = String::with_capacity(
+        (longest_package_name_length
+            + longest_current_version_length
+            + longest_major_version_length
+            + 5)
+            * total_lines,
+    );
+
+    output_string.push_str("\nThe following dependencies have new major versions available:\n\n");
+    for (name, v1, v2) in versions {
+        let name_padding = " ".repeat(longest_package_name_length - name.len());
+        let curr_ver_padding = " ".repeat(longest_current_version_length - v1.to_string().len());
+
+        output_string.push_str(
+            &[
+                name,
+                &name_padding,
+                " ",
+                &v1.to_string(),
+                &curr_ver_padding,
+                " -> ",
+                &v2.to_string(),
+                "\n",
+            ]
+            .concat(),
+        );
     }
-    LocalPackages::from_manifest(&manifest).write_to_disc(paths)?;
 
-    Ok(manifest)
+    output_string
 }
 
 async fn add_missing_packages<Telem: Telemetry>(
@@ -601,62 +618,6 @@ impl LocalPackages {
     }
 }
 
-fn get_manifest<Telem: Telemetry>(
-    paths: &ProjectPaths,
-    runtime: tokio::runtime::Handle,
-    mode: Mode,
-    config: &PackageConfig,
-    telemetry: &Telem,
-    use_manifest: UseManifest,
-    packages_to_update: Vec<EcoString>,
-) -> Result<(bool, Manifest)> {
-    // If there's no manifest (or we have been asked not to use it) then resolve
-    // the versions anew
-    let should_resolve = match use_manifest {
-        _ if !paths.manifest().exists() => {
-            tracing::debug!("manifest_not_present");
-            true
-        }
-        UseManifest::No => {
-            tracing::debug!("ignoring_manifest");
-            true
-        }
-        UseManifest::Yes => false,
-    };
-
-    if should_resolve {
-        let manifest = resolve_versions(runtime, mode, paths, config, None, telemetry, Vec::new())?;
-        return Ok((true, manifest));
-    }
-
-    let manifest = read_manifest_from_disc(paths)?;
-
-    // If there are no requested updates, and the config is unchanged
-    // since the manifest was written then it is up to date so we can return it unmodified.
-    if packages_to_update.is_empty()
-        && is_same_requirements(
-            &manifest.requirements,
-            &config.all_direct_dependencies()?,
-            paths.root(),
-        )?
-    {
-        tracing::debug!("manifest_up_to_date");
-        Ok((false, manifest))
-    } else {
-        tracing::debug!("manifest_outdated");
-        let manifest = resolve_versions(
-            runtime,
-            mode,
-            paths,
-            config,
-            Some(&manifest),
-            telemetry,
-            packages_to_update,
-        )?;
-        Ok((true, manifest))
-    }
-}
-
 fn is_same_requirements(
     requirements1: &HashMap<EcoString, Requirement>,
     requirements2: &HashMap<EcoString, Requirement>,
@@ -799,81 +760,6 @@ impl PartialEq for ProvidedPackageSource {
     }
 }
 
-fn resolve_versions<Telem: Telemetry>(
-    runtime: tokio::runtime::Handle,
-    mode: Mode,
-    project_paths: &ProjectPaths,
-    config: &PackageConfig,
-    manifest: Option<&Manifest>,
-    telemetry: &Telem,
-    packages_to_update: Vec<EcoString>,
-) -> Result<Manifest, Error> {
-    telemetry.resolving_package_versions();
-    let dependencies = config.dependencies_for(mode)?;
-    let mut locked = config.locked(manifest)?;
-
-    if !packages_to_update.is_empty() {
-        unlock_packages(&mut locked, &packages_to_update, manifest)?;
-    }
-
-    // Packages which are provided directly instead of downloaded from hex
-    let mut provided_packages = HashMap::new();
-    // The version requires of the current project
-    let mut root_requirements = HashMap::new();
-
-    // Populate the provided_packages and root_requirements maps
-    for (name, requirement) in dependencies.into_iter() {
-        let version = match requirement {
-            Requirement::Hex { version } => version,
-            Requirement::Path { path } => provide_local_package(
-                name.clone(),
-                &path,
-                project_paths.root(),
-                project_paths,
-                &mut provided_packages,
-                &mut vec![],
-            )?,
-            Requirement::Git { git, ref_ } => provide_git_package(
-                name.clone(),
-                &git,
-                &ref_,
-                project_paths,
-                &mut provided_packages,
-                &mut Vec::new(),
-            )?,
-        };
-        let _ = root_requirements.insert(name, version);
-    }
-
-    // Convert provided packages into hex packages for pub-grub resolve
-    let provided_hex_packages = provided_packages
-        .iter()
-        .map(|(name, package)| (name.clone(), package.to_hex_package(name)))
-        .collect();
-
-    let resolved = dependency::resolve_versions(
-        PackageFetcher::boxed(runtime.clone()),
-        provided_hex_packages,
-        config.name.clone(),
-        root_requirements.into_iter(),
-        &locked,
-    )?;
-
-    // Convert the hex packages and local packages into manifest packages
-    let manifest_packages = runtime.block_on(future::try_join_all(
-        resolved
-            .into_iter()
-            .map(|(name, version)| lookup_package(name, version, &provided_packages)),
-    ))?;
-
-    let manifest = Manifest {
-        packages: manifest_packages,
-        requirements: config.all_direct_dependencies()?,
-    };
-
-    Ok(manifest)
-}
-
 /// Provide a package from a local project
 fn provide_local_package(
     package_name: EcoString,
@@ -1075,7 +961,8 @@ fn provide_package(
     match provided.get(&package_name) {
         Some(package) if package.source == package_source => {
             // This package has already been provided from this source, return the version
-            let version = hexpm::version::Range::new(format!("== {}", &package.version));
+            let version = hexpm::version::Range::new(format!("== {}", &package.version))
+                .expect("== {version} should be a valid range");
             return Ok(version);
         }
         Some(package) => {
@@ -1123,7 +1010,8 @@ fn provide_package(
     }
     let _ = parents.pop();
     // Add the package to the provided packages dictionary
-    let version = hexpm::version::Range::new(format!("== {}", &config.version));
+    let version = hexpm::version::Range::new(format!("== {}", &config.version))
+        .expect("== {version} should be a valid range");
     let _ = provided.insert(
         config.name,
         ProvidedPackage {
@@ -1227,16 +1115,26 @@ async fn lookup_package(
 }
 
 struct PackageFetcher {
+    runtime_cache: RefCell<HashMap<String, Rc<hexpm::Package>>>,
     runtime: tokio::runtime::Handle,
     http: HttpClient,
 }
 
 impl PackageFetcher {
-    pub fn boxed(runtime: tokio::runtime::Handle) -> Box<Self> {
-        Box::new(Self {
+    pub fn new(runtime: tokio::runtime::Handle) -> Self {
+        Self {
+            runtime_cache: RefCell::new(HashMap::new()),
             runtime,
             http: HttpClient::new(),
-        })
+        }
+    }
+
+    /// Caches the result of `get_dependencies` so that we don't need to make a network request.
+    /// Currently dependencies are fetched during initial version resolution, and then during check
+    /// for major version availability.
+    fn cache_package(&self, package: &str, result: Rc<hexpm::Package>) {
+        let mut runtime_cache = self.runtime_cache.borrow_mut();
+        let _ = runtime_cache.insert(package.to_string(), result);
     }
 }
 
@@ -1267,26 +1165,29 @@ impl TarUnpacker for Untar {
 }
 
 impl dependency::PackageFetcher for PackageFetcher {
-    fn get_dependencies(
-        &self,
-        package: &str,
-    ) -> Result<hexpm::Package, Box<dyn std::error::Error>> {
+    fn get_dependencies(&self, package: &str) -> Result<Rc<hexpm::Package>, PackageFetchError> {
+        {
+            let runtime_cache = self.runtime_cache.borrow();
+            let result = runtime_cache.get(package);
+
+            if let Some(result) = result {
+                return Ok(result.clone());
+            }
+        }
+
         tracing::debug!(package = package, "looking_up_hex_package");
         let config = hexpm::Config::new();
         let request = hexpm::get_package_request(package, None, &config);
         let response = self
             .runtime
             .block_on(self.http.send(request))
-            .map_err(Box::new)?;
+            .map_err(PackageFetchError::fetch_error)?;
 
-        match hexpm::get_package_response(response, HEXPM_PUBLIC_KEY) {
-            Ok(a) => Ok(a),
-            Err(e) => match e {
-                hexpm::ApiError::NotFound => {
-                    Err(format!("I couldn't find a package called `{}`", package).into())
-                }
-                _ => Err(e.into()),
-            },
-        }
+        let pkg = hexpm::get_package_response(response, HEXPM_PUBLIC_KEY)
+            .map_err(PackageFetchError::from)?;
+        let pkg = Rc::new(pkg);
+        let pkg_ref = Rc::clone(&pkg);
+        self.cache_package(package, pkg);
+        Ok(pkg_ref)
     }
 }
