@@ -8,7 +8,6 @@
     clippy::needless_continue,
     clippy::needless_borrow,
     clippy::match_wildcard_for_single_variants,
-    clippy::match_on_vec_items,
     clippy::imprecise_flops,
     clippy::suboptimal_flops,
     clippy::lossy_float_literal,
@@ -76,7 +75,6 @@ pub mod run;
 mod shell;
 
 use config::root_config;
-use dependencies::UseManifest;
 use fs::{get_current_directory, get_project_root};
 pub use gleam_core::error::{Error, Result};
 
@@ -781,7 +779,10 @@ fn download_dependencies(paths: &ProjectPaths) -> Result<()> {
         cli::Reporter::new(),
         None,
         Vec::new(),
-        UseManifest::Yes,
+        dependencies::DependencyManagerConfig {
+            use_manifest: dependencies::UseManifest::Yes,
+            check_major_versions: dependencies::CheckMajorVersions::No,
+        },
     )?;
     Ok(())
 }
