@@ -107,18 +107,18 @@ pub fn command(paths: &ProjectPaths, replace: bool, i_am_sure: bool) -> Result<(
     );
 
     // Prompt the user to make a git tag if they have not.
-    let has_repo = config.repository.url().is_some();
+    let has_repo = config.repository.is_some();
     let git = PathBuf::from(".git");
-    let version = format!("v{}", &config.version);
-    let git_tag = git.join("refs").join("tags").join(&version);
+    let tag_name = config.tag_for_version(&config.version);
+    let git_tag = git.join("refs").join("tags").join(&tag_name);
     if has_repo && git.exists() && !git_tag.exists() {
         println!(
             "
 Please push a git tag for this release so source code links in the
 HTML documentation will work:
 
-    git tag {version}
-    git push origin {version}
+    git tag {tag_name}
+    git push origin {tag_name}
 "
         )
     }
@@ -207,9 +207,10 @@ For example:
 }
 
 fn check_repo_url(config: &PackageConfig, i_am_sure: bool) -> Result<bool, Error> {
-    let Some(url) = config.repository.url() else {
+    let Some(repo) = config.repository.as_ref() else {
         return Ok(true);
     };
+    let url = repo.url();
 
     let runtime = tokio::runtime::Runtime::new().expect("Unable to start Tokio async runtime");
     let response = runtime.block_on(reqwest::get(&url)).map_err(Error::http)?;
@@ -335,8 +336,7 @@ fn do_build_hex_tarball(paths: &ProjectPaths, config: &mut PackageConfig) -> Res
             // inferred lower bound could be lower.
             let minimum_required_version =
                 std::cmp::max(minimum_required_version, Version::new(1, 0, 0));
-            let inferred_version_range =
-                pubgrub::range::Range::higher_than(minimum_required_version);
+            let inferred_version_range = pubgrub::Range::higher_than(minimum_required_version);
             config.gleam_version = Some(GleamVersion::from_pubgrub(inferred_version_range));
         }
         // Otherwise we need to check that the annotated version range is
@@ -444,7 +444,14 @@ fn metadata_config<'a>(
     source_files: &[Utf8PathBuf],
     generated_files: &[(Utf8PathBuf, String)],
 ) -> Result<String> {
-    let repo_url = http::Uri::try_from(config.repository.url().unwrap_or_default()).ok();
+    let repo_url = http::Uri::try_from(
+        config
+            .repository
+            .as_ref()
+            .map(|r| r.url())
+            .unwrap_or_default(),
+    )
+    .ok();
     let requirements: Result<Vec<ReleaseRequirement<'a>>> = config
         .dependencies
         .iter()
@@ -699,8 +706,8 @@ fn release_metadata_as_erlang() {
     let version = "1.2.3".try_into().unwrap();
     let homepage = "https://gleam.run".parse().unwrap();
     let github = "https://github.com/lpil/myapp".parse().unwrap();
-    let req1 = Range::new("~> 1.2.3 or >= 5.0.0".into());
-    let req2 = Range::new("~> 1.2".into());
+    let req1 = Range::new("~> 1.2.3 or >= 5.0.0".into()).unwrap();
+    let req2 = Range::new("~> 1.2".into()).unwrap();
     let meta = ReleaseMetadata {
         name: "myapp",
         version: &version,
