@@ -8,7 +8,7 @@ use crate::{
     GLEAM_CORE_PACKAGE_NAME,
     ast::{
         self, Arg, BitArrayOption, CustomType, Definition, DefinitionLocation, Function,
-        GroupedStatements, Import, ModuleConstant, Publicity, RecordConstructor,
+        GroupedDefinitions, Import, ModuleConstant, Publicity, RecordConstructor,
         RecordConstructorArg, SrcSpan, Statement, TypeAlias, TypeAst, TypeAstConstructor,
         TypeAstFn, TypeAstHole, TypeAstTuple, TypeAstVar, TypedDefinition, TypedExpr,
         TypedFunction, TypedModule, UntypedArg, UntypedCustomType, UntypedFunction, UntypedImport,
@@ -214,7 +214,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 .package_config
                 .gleam_version
                 .clone()
-                .map(|version| version.as_pubgrub()),
+                .map(|version| version.into()),
             current_module: self.module_name.clone(),
             target: self.target,
             importable_modules: self.importable_modules,
@@ -223,76 +223,85 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         }
         .build();
 
-        let statements = GroupedStatements::new(module.into_iter_statements(self.target));
-        let statements_count = statements.len();
+        let definitions = GroupedDefinitions::new(module.into_iter_definitions(self.target));
+        let definitions_count = definitions.len();
 
         // Register any modules, types, and values being imported
         // We process imports first so that anything imported can be referenced
         // anywhere in the module.
-        let mut env = Importer::run(self.origin, env, &statements.imports, &mut self.problems);
+        let mut env = Importer::run(self.origin, env, &definitions.imports, &mut self.problems);
 
         // Register types so they can be used in constructors and functions
         // earlier in the module.
-        for t in &statements.custom_types {
-            if let Err(error) = self.register_types_from_custom_type(t, &mut env) {
+        for type_ in &definitions.custom_types {
+            if let Err(error) = self.register_types_from_custom_type(type_, &mut env) {
                 return self.all_errors(error);
             }
         }
 
-        let sorted_aliases = match sorted_type_aliases(&statements.type_aliases) {
-            Ok(it) => it,
+        let sorted_aliases = match sorted_type_aliases(&definitions.type_aliases) {
+            Ok(sorted_aliases) => sorted_aliases,
             Err(error) => return self.all_errors(error),
         };
-        for t in sorted_aliases {
-            self.register_type_alias(t, &mut env);
+        for type_alias in sorted_aliases {
+            self.register_type_alias(type_alias, &mut env);
         }
 
-        for f in &statements.functions {
-            self.register_value_from_function(f, &mut env);
+        for function in &definitions.functions {
+            self.register_value_from_function(function, &mut env);
         }
 
         // Infer the types of each statement in the module
-        let mut typed_statements = Vec::with_capacity(statements_count);
-        for i in statements.imports {
-            optionally_push(&mut typed_statements, self.analyse_import(i, &env));
+        let mut typed_definitions = Vec::with_capacity(definitions_count);
+        for import in definitions.imports {
+            optionally_push(&mut typed_definitions, self.analyse_import(import, &env));
         }
-        for t in statements.custom_types {
-            optionally_push(&mut typed_statements, self.analyse_custom_type(t, &mut env));
+        for type_ in definitions.custom_types {
+            optionally_push(
+                &mut typed_definitions,
+                self.analyse_custom_type(type_, &mut env),
+            );
         }
-        for t in statements.type_aliases {
-            typed_statements.push(analyse_type_alias(t, &mut env));
+        for type_alias in definitions.type_aliases {
+            typed_definitions.push(analyse_type_alias(type_alias, &mut env));
         }
 
-        // Sort functions and constants into dependency order for inference. Definitions that do
-        // not depend on other definitions are inferred first, then ones that depend
-        // on those, etc.
+        // Sort functions and constants into dependency order for inference.
+        // Definitions that do not depend on other definitions are inferred
+        // first, then ones that depend on those, etc.
         let definition_groups =
-            match into_dependency_order(statements.functions, statements.constants) {
-                Ok(it) => it,
+            match into_dependency_order(definitions.functions, definitions.constants) {
+                Ok(definition_groups) => definition_groups,
                 Err(error) => return self.all_errors(error),
             };
         let mut working_group = vec![];
 
         for group in definition_groups {
-            // A group may have multiple functions that depend on each other through
-            // mutual recursion.
+            // A group may have multiple functions that depend on each other
+            // through mutual recursion.
 
             for definition in group {
-                let def = match definition {
-                    CallGraphNode::Function(f) => self.infer_function(f, &mut env),
-                    CallGraphNode::ModuleConstant(c) => self.infer_module_constant(c, &mut env),
+                let definition = match definition {
+                    CallGraphNode::Function(function) => self.infer_function(function, &mut env),
+                    CallGraphNode::ModuleConstant(constant) => {
+                        self.infer_module_constant(constant, &mut env)
+                    }
                 };
-                working_group.push(def);
+                working_group.push(definition);
             }
 
             // Now that the entire group has been inferred, generalise their types.
             for inferred in working_group.drain(..) {
-                typed_statements.push(generalise_statement(inferred, &self.module_name, &mut env));
+                typed_definitions.push(generalise_definition(
+                    inferred,
+                    &self.module_name,
+                    &mut env,
+                ));
             }
         }
 
         // Generate warnings for unused items
-        env.convert_unused_to_warnings(&mut self.problems);
+        let unused_definition_positions = env.handle_unused(&mut self.problems);
 
         // Remove imported types and values to create the public interface
         // Private types and values are retained so they can be used in the language
@@ -335,7 +344,9 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         let module = ast::Module {
             documentation: documentation.clone(),
             name: self.module_name.clone(),
-            definitions: typed_statements,
+            definitions: typed_definitions,
+            names: type_names,
+            unused_definition_positions,
             type_info: ModuleInterface {
                 name: self.module_name,
                 types,
@@ -362,7 +373,6 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                     type_references: env.references.type_references,
                 },
             },
-            names: type_names,
         };
 
         match Vec1::try_from_vec(self.problems.take_errors()) {
@@ -392,6 +402,8 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             ..
         } = c;
         self.check_name_case(name_location, &name, Named::Constant);
+        // If the constant's name matches an unqualified import, emit a warning:
+        self.check_shadow_import(&name, c.location, environment);
 
         environment.references.begin_constant();
 
@@ -486,6 +498,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             publicity,
             arguments,
             body,
+            body_start,
             return_annotation,
             end_position: end_location,
             deprecation,
@@ -503,7 +516,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             .expect("Could not find preregistered type for function");
         let field_map = preregistered_fn.field_map().cloned();
         let preregistered_type = preregistered_fn.type_.clone();
-        let (prereg_args_types, prereg_return_type) = preregistered_type
+        let (prereg_arguments_types, prereg_return_type) = preregistered_type
             .fn_types()
             .expect("Preregistered type for fn was not a fn");
 
@@ -538,11 +551,23 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             has_javascript_external: external_javascript.is_some(),
         };
 
-        let typed_args = arguments
-            .into_iter()
-            .zip(&prereg_args_types)
-            .map(|(a, t)| a.set_type(t.clone()))
-            .collect_vec();
+        let mut typed_arguments = Vec::with_capacity(arguments.len());
+        for (argument, type_) in arguments.into_iter().zip(&prereg_arguments_types) {
+            let argument = argument.set_type(type_.clone());
+            match &argument.names {
+                ast::ArgNames::Named { .. } | ast::ArgNames::NamedLabelled { .. } => (),
+                ast::ArgNames::Discard { name, location }
+                | ast::ArgNames::LabelledDiscard {
+                    name,
+                    name_location: location,
+                    ..
+                } => {
+                    let _ = environment.discarded_names.insert(name.clone(), *location);
+                }
+            }
+
+            typed_arguments.push(argument);
+        }
 
         // We have already registered the function in the `register_value_from_function`
         // method, but here we must set this as the current function again, so that anything
@@ -557,13 +582,13 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 .remove(&name)
                 .expect("Could not find hydrator for fn");
 
-            let (args, body) = expr_typer.infer_fn_with_known_types(
-                typed_args.clone(),
+            let (arguments, body) = expr_typer.infer_fn_with_known_types(
+                typed_arguments.clone(),
                 body,
                 Some(prereg_return_type.clone()),
             )?;
-            let args_types = args.iter().map(|a| a.type_.clone()).collect();
-            let type_ = fn_(args_types, body.last().type_());
+            let arguments_types = arguments.iter().map(|a| a.type_.clone()).collect();
+            let type_ = fn_(arguments_types, body.last().type_());
             Ok((
                 type_,
                 body,
@@ -661,7 +686,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 .map(|(m, f, _)| (m.clone(), f.clone())),
             field_map,
             module: environment.current_module.clone(),
-            arity: typed_args.len(),
+            arity: typed_arguments.len(),
             location,
             implementations,
             purity,
@@ -689,7 +714,8 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             name: Some((name_location, name)),
             publicity,
             deprecation,
-            arguments: typed_args,
+            arguments: typed_arguments,
+            body_start,
             end_position: end_location,
             return_annotation,
             return_type: preregistered_type
@@ -885,7 +911,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                      location,
                      name_location,
                      name,
-                     arguments: args,
+                     arguments,
                      documentation,
                      deprecation: constructor_deprecation,
                  }| {
@@ -902,11 +928,11 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                         .expect("Could not find preregistered type for function");
                     let preregistered_type = preregistered_fn.type_.clone();
 
-                    let args = match preregistered_type.fn_types() {
-                        Some((args_types, _return_type)) => args
+                    let arguments = match preregistered_type.fn_types() {
+                        Some((arguments_types, _return_type)) => arguments
                             .into_iter()
-                            .zip(&args_types)
-                            .map(|(argument, t)| {
+                            .zip(&arguments_types)
+                            .map(|(argument, type_)| {
                                 if let Some((location, label)) = &argument.label {
                                     self.check_name_case(*location, label, Named::Label);
                                 }
@@ -915,7 +941,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                                     label: argument.label,
                                     ast: argument.ast,
                                     location: argument.location,
-                                    type_: t.clone(),
+                                    type_: type_.clone(),
                                     doc: argument.doc,
                                 }
                             })
@@ -929,7 +955,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                         location,
                         name_location,
                         name,
-                        arguments: args,
+                        arguments,
                         documentation,
                         deprecation: constructor_deprecation,
                     }
@@ -1047,7 +1073,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 .register_type_reference_in_call_graph(name.clone());
 
             let mut field_map_builder = FieldMapBuilder::new(constructor.arguments.len() as u32);
-            let mut args_types = Vec::with_capacity(constructor.arguments.len());
+            let mut arguments_types = Vec::with_capacity(constructor.arguments.len());
             let mut fields = Vec::with_capacity(constructor.arguments.len());
 
             for RecordConstructorArg {
@@ -1072,7 +1098,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 });
 
                 // Register the type for this parameter
-                args_types.push(t);
+                arguments_types.push(t);
 
                 let (label_location, label) = match label {
                     Some((location, label)) => (*location, Some(label)),
@@ -1090,7 +1116,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             type_.set_custom_type_variant(index as u16);
             let type_ = match constructor.arguments.len() {
                 0 => Arc::new(type_),
-                _ => fn_(args_types.clone(), Arc::new(type_)),
+                _ => fn_(arguments_types.clone(), Arc::new(type_)),
             };
             let constructor_info = ValueConstructorVariant::Record {
                 documentation: constructor
@@ -1242,7 +1268,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             package: environment.current_package.clone(),
             module: self.module_name.to_owned(),
             name: name.clone(),
-            args: parameters.clone(),
+            arguments: parameters.clone(),
             inferred_variant: None,
         });
         let _ = self.hydrators.insert(name.clone(), hydrator);
@@ -1292,7 +1318,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         let TypeAlias {
             location,
             publicity,
-            parameters: args,
+            parameters: arguments,
             alias: name,
             name_location,
             type_ast: resolved_type,
@@ -1318,7 +1344,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         // Use the hydrator to convert the AST into a type, erroring if the AST was invalid
         // in some fashion.
         let mut hydrator = Hydrator::new();
-        let parameters = self.make_type_vars(args, &mut hydrator, environment);
+        let parameters = self.make_type_vars(arguments, &mut hydrator, environment);
         let arity = parameters.len();
         let tryblock = || {
             hydrator.disallow_new_type_variables();
@@ -1370,11 +1396,12 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
 
     fn make_type_vars(
         &mut self,
-        args: &[SpannedString],
+        arguments: &[SpannedString],
         hydrator: &mut Hydrator,
         environment: &mut Environment<'_>,
     ) -> Vec<Arc<Type>> {
-        args.iter()
+        arguments
+            .iter()
             .map(|(location, name)| {
                 self.check_name_case(*location, name, Named::TypeVariable);
                 match hydrator.add_type_variable(name, environment) {
@@ -1404,7 +1431,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
     ) {
         let Function {
             name,
-            arguments: args,
+            arguments,
             location,
             return_annotation,
             publicity,
@@ -1414,6 +1441,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             deprecation,
             end_position: _,
             body: _,
+            body_start: _,
             return_type: _,
             implementations,
             purity,
@@ -1421,6 +1449,8 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         let (name_location, name) = name.as_ref().expect("A module's function must be named");
 
         self.check_name_case(*name_location, name, Named::Function);
+        // If the function's name matches an unqualified import, emit a warning:
+        self.check_shadow_import(name, f.location, environment);
 
         environment.references.register_value(
             name.clone(),
@@ -1429,10 +1459,10 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             *publicity,
         );
 
-        let mut builder = FieldMapBuilder::new(args.len() as u32);
+        let mut builder = FieldMapBuilder::new(arguments.len() as u32);
         for Arg {
             names, location, ..
-        } in args.iter()
+        } in arguments.iter()
         {
             check_argument_names(names, &mut self.problems);
 
@@ -1447,11 +1477,11 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
         // must be given in full, so we disallow holes in the annotations.
         hydrator.permit_holes(external_erlang.is_none() && external_javascript.is_none());
 
-        let arg_types = args
+        let arguments_types = arguments
             .iter()
-            .map(|arg| {
+            .map(|argument| {
                 match hydrator.type_from_option_ast(
-                    &arg.annotation,
+                    &argument.annotation,
                     environment,
                     &mut self.problems,
                 ) {
@@ -1474,7 +1504,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 }
             };
 
-        let type_ = fn_(arg_types, return_type);
+        let type_ = fn_(arguments_types, return_type);
         let _ = self.hydrators.insert(name.clone(), hydrator);
 
         let variant = ValueConstructorVariant::ModuleFn {
@@ -1488,7 +1518,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 .as_ref()
                 .map(|(m, f, _)| (m.clone(), f.clone())),
             module: environment.current_module.clone(),
-            arity: args.len(),
+            arity: arguments.len(),
             location: *location,
             implementations: *implementations,
             purity: *purity,
@@ -1550,6 +1580,21 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             self.minimum_required_version = minimum_required_version;
         }
     }
+
+    fn check_shadow_import(
+        &mut self,
+        name: &EcoString,
+        location: SrcSpan,
+        environment: &mut Environment<'_>,
+    ) {
+        if environment.unqualified_imported_names.contains_key(name) {
+            self.problems
+                .warning(Warning::TopLevelDefinitionShadowsImport {
+                    location,
+                    name: name.clone(),
+                });
+        }
+    }
 }
 
 fn optionally_push<T>(vector: &mut Vec<T>, item: Option<T>) {
@@ -1591,7 +1636,7 @@ fn analyse_type_alias(t: UntypedTypeAlias, environment: &mut Environment<'_>) ->
         publicity,
         alias,
         name_location,
-        parameters: args,
+        parameters: arguments,
         type_ast: resolved_type,
         deprecation,
         ..
@@ -1611,7 +1656,7 @@ fn analyse_type_alias(t: UntypedTypeAlias, environment: &mut Environment<'_>) ->
         publicity,
         alias,
         name_location,
-        parameters: args,
+        parameters: arguments,
         type_ast: resolved_type,
         type_,
         deprecation,
@@ -1666,19 +1711,19 @@ where
     }
 }
 
-fn generalise_statement(
-    s: TypedDefinition,
+fn generalise_definition(
+    definition: TypedDefinition,
     module_name: &EcoString,
     environment: &mut Environment<'_>,
 ) -> TypedDefinition {
-    match s {
+    match definition {
         Definition::Function(function) => generalise_function(function, environment, module_name),
         Definition::ModuleConstant(constant) => {
             generalise_module_constant(constant, environment, module_name)
         }
-        statement @ (Definition::TypeAlias(TypeAlias { .. })
+        definition @ (Definition::TypeAlias(TypeAlias { .. })
         | Definition::CustomType(CustomType { .. })
-        | Definition::Import(Import { .. })) => statement,
+        | Definition::Import(Import { .. })) => definition,
     }
 }
 
@@ -1752,10 +1797,11 @@ fn generalise_function(
         name,
         publicity,
         deprecation,
-        arguments: args,
+        arguments,
         body,
         return_annotation,
         end_position: end_location,
+        body_start,
         return_type,
         external_erlang,
         external_javascript,
@@ -1786,7 +1832,7 @@ fn generalise_function(
             .as_ref()
             .map(|(m, f, _)| (m.clone(), f.clone())),
         module: module_name.clone(),
-        arity: args.len(),
+        arity: arguments.len(),
         location,
         implementations,
         purity,
@@ -1814,8 +1860,9 @@ fn generalise_function(
         name: Some((name_location, name)),
         publicity,
         deprecation,
-        arguments: args,
+        arguments,
         end_position: end_location,
+        body_start,
         return_annotation,
         return_type,
         body,
@@ -1847,11 +1894,11 @@ struct Accessors {
 }
 
 fn custom_type_accessors(constructors: &[TypeValueConstructor]) -> Result<Accessors, Error> {
-    let args = get_compatible_record_fields(constructors);
+    let arguments = get_compatible_record_fields(constructors);
 
-    let mut shared_accessors = HashMap::with_capacity(args.len());
+    let mut shared_accessors = HashMap::with_capacity(arguments.len());
 
-    for (index, label, type_) in args {
+    for (index, label, type_) in arguments {
         let _ = shared_accessors.insert(
             label.clone(),
             RecordAccessor {
