@@ -1,12 +1,13 @@
 use std::sync::Arc;
 
 use camino::Utf8PathBuf;
+use ecow::EcoString;
 
 use crate::analyse::TargetSupport;
 use crate::build::{ExpressionPosition, Origin, Target};
 use crate::config::PackageConfig;
 use crate::line_numbers::LineNumbers;
-use crate::type_::error::VariableOrigin;
+use crate::type_::error::{VariableDeclaration, VariableOrigin, VariableSyntax};
 use crate::type_::expression::{FunctionDefinition, Purity};
 use crate::type_::{Deprecation, PRELUDE_MODULE_NAME, Problems};
 use crate::warning::WarningEmitter;
@@ -93,7 +94,7 @@ fn compile_expression(src: &str) -> TypedStatement {
         package: "mypackage".into(),
         module: "mymod".into(),
         name: "Cat".into(),
-        args: vec![],
+        arguments: vec![],
         inferred_variant: None,
     });
     let variant = ValueConstructorVariant::Record {
@@ -193,6 +194,12 @@ fn find_node_todo() {
 fn find_node_todo_with_string() {
     let statement = compile_expression(r#" todo as "ok" "#);
     let expr = get_bare_expression(&statement);
+    let message = TypedExpr::String {
+        location: SrcSpan { start: 9, end: 13 },
+        type_: type_::string(),
+        value: "ok".into(),
+    };
+
     assert_eq!(expr.find_node(0), None);
     assert_eq!(
         expr.find_node(1),
@@ -204,14 +211,14 @@ fn find_node_todo_with_string() {
     assert_eq!(
         expr.find_node(12),
         Some(Located::Expression {
-            expression: expr,
+            expression: &message,
             position: ExpressionPosition::Expression
         })
     );
     assert_eq!(
         expr.find_node(13),
         Some(Located::Expression {
-            expression: expr,
+            expression: &message,
             position: ExpressionPosition::Expression
         })
     );
@@ -327,7 +334,10 @@ wibble}"#,
             publicity: Publicity::Private,
             variant: ValueConstructorVariant::LocalVariable {
                 location: SrcSpan { start: 5, end: 11 },
-                origin: VariableOrigin::Variable("wibble".into()),
+                origin: VariableOrigin {
+                    syntax: VariableSyntax::Variable("wibble".into()),
+                    declaration: VariableDeclaration::LetPattern,
+                },
             },
             type_: type_::int(),
         },
@@ -753,7 +763,7 @@ fn find_node_call() {
 
     let TypedExpr::Call {
         fun: called_function,
-        args: function_arguments,
+        arguments: function_arguments,
         ..
     } = expr
     else {
@@ -884,6 +894,32 @@ fn find_node_record_update() {
     let statement = compile_expression(r#"Cat(..Cat("Nubi", 3), age: 4)"#);
     let update = get_bare_expression(&statement);
 
+    let cat = TypedExpr::Var {
+        location: SrcSpan { start: 0, end: 3 },
+        constructor: ValueConstructor {
+            publicity: Publicity::Public,
+            deprecation: Deprecation::NotDeprecated,
+            variant: ValueConstructorVariant::Record {
+                name: "Cat".into(),
+                arity: 2,
+                field_map: Some(FieldMap {
+                    arity: 2,
+                    fields: [(EcoString::from("age"), 1), (EcoString::from("name"), 0)].into(),
+                }),
+                location: SrcSpan { start: 12, end: 15 },
+                module: "mymod".into(),
+                variants_count: 1,
+                variant_index: 0,
+                documentation: Some("wibble".into()),
+            },
+            type_: type_::fn_(
+                vec![type_::string(), type_::int()],
+                type_::named("mypackage", "mymod", "Cat", Publicity::Public, vec![]),
+            ),
+        },
+        name: "Cat".into(),
+    };
+
     let int = TypedExpr::Int {
         location: SrcSpan { start: 27, end: 28 },
         value: "4".into(),
@@ -894,14 +930,14 @@ fn find_node_record_update() {
     assert_eq!(
         update.find_node(0),
         Some(Located::Expression {
-            expression: update,
+            expression: &cat,
             position: ExpressionPosition::Expression
         })
     );
     assert_eq!(
         update.find_node(3),
         Some(Located::Expression {
-            expression: update,
+            expression: &cat,
             position: ExpressionPosition::Expression
         })
     );
