@@ -58,7 +58,7 @@ pub enum TypedExpr {
         location: SrcSpan,
         type_: Arc<Type>,
         kind: FunctionLiteralKind,
-        args: Vec<TypedArg>,
+        arguments: Vec<TypedArg>,
         body: Vec1<TypedStatement>,
         return_annotation: Option<TypeAst>,
         purity: Purity,
@@ -75,7 +75,7 @@ pub enum TypedExpr {
         location: SrcSpan,
         type_: Arc<Type>,
         fun: Box<Self>,
-        args: Vec<CallArg<Self>>,
+        arguments: Vec<CallArg<Self>>,
     },
 
     BinOp {
@@ -144,6 +144,7 @@ pub enum TypedExpr {
         location: SrcSpan,
         type_: Arc<Type>,
         expression: Option<Box<Self>>,
+        message: Option<Box<Self>>,
     },
 
     BitArray {
@@ -164,9 +165,11 @@ pub enum TypedExpr {
     RecordUpdate {
         location: SrcSpan,
         type_: Arc<Type>,
-        record: Box<TypedAssignment>,
+        /// If the record is an expression that is not a variable we will need to assign to a
+        /// variable so it can be referred multiple times.
+        record_assignment: Option<Box<TypedAssignment>>,
         constructor: Box<Self>,
-        args: Vec<CallArg<Self>>,
+        arguments: Vec<CallArg<Self>>,
     },
 
     NegateBool {
@@ -190,7 +193,7 @@ pub enum TypedExpr {
 impl TypedExpr {
     pub fn is_println(&self) -> bool {
         let fun = match self {
-            TypedExpr::Call { fun, args, .. } if args.len() == 1 => fun.as_ref(),
+            TypedExpr::Call { fun, arguments, .. } if arguments.len() == 1 => fun.as_ref(),
             _ => return false,
         };
 
@@ -206,7 +209,6 @@ impl TypedExpr {
         match self {
             Self::Var { .. }
             | Self::Int { .. }
-            | Self::Panic { .. }
             | Self::Float { .. }
             | Self::String { .. }
             | Self::Invalid { .. } => self.self_if_contains_location(byte_index),
@@ -240,13 +242,30 @@ impl TypedExpr {
                 }
             }
 
-            Self::Echo { expression, .. } => expression
+            Self::Echo {
+                expression,
+                message,
+                ..
+            } => expression
                 .as_ref()
                 .and_then(|expression| expression.find_node(byte_index))
+                .or_else(|| {
+                    message
+                        .as_ref()
+                        .and_then(|message| message.find_node(byte_index))
+                })
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
-            Self::Todo { kind, .. } => match kind {
-                TodoKind::Keyword => self.self_if_contains_location(byte_index),
+            Self::Panic { message, .. } => message
+                .as_ref()
+                .and_then(|message| message.find_node(byte_index))
+                .or_else(|| self.self_if_contains_location(byte_index)),
+
+            Self::Todo { kind, message, .. } => match kind {
+                TodoKind::Keyword => message
+                    .as_ref()
+                    .and_then(|message| message.find_node(byte_index))
+                    .or_else(|| self.self_if_contains_location(byte_index)),
                 // We don't want to match on todos that were implicitly inserted
                 // by the compiler as it would result in confusing suggestions
                 // from the LSP.
@@ -332,15 +351,17 @@ impl TypedExpr {
                 .find_node(byte_index)
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
-            Self::Fn { body, args, .. } => args
+            Self::Fn {
+                body, arguments, ..
+            } => arguments
                 .iter()
                 .find_map(|arg| arg.find_node(byte_index))
                 .or_else(|| body.iter().find_map(|s| s.find_node(byte_index)))
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
-            Self::Call { fun, args, .. } => args
+            Self::Call { fun, arguments, .. } => arguments
                 .iter()
-                .find_map(|arg| arg.find_node(byte_index, fun, args))
+                .find_map(|argument| argument.find_node(byte_index, fun, arguments))
                 .or_else(|| fun.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
@@ -371,15 +392,20 @@ impl TypedExpr {
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
             Self::RecordUpdate {
-                record,
+                record_assignment,
                 constructor,
-                args,
+                arguments,
                 ..
-            } => args
+            } => arguments
                 .iter()
-                .filter(|arg| arg.implicit.is_none())
-                .find_map(|arg| arg.find_node(byte_index, constructor, args))
-                .or_else(|| record.find_node(byte_index))
+                .filter(|argument| argument.implicit.is_none())
+                .find_map(|argument| argument.find_node(byte_index, constructor, arguments))
+                .or_else(|| constructor.find_node(byte_index))
+                .or_else(|| {
+                    record_assignment
+                        .as_ref()
+                        .and_then(|assignment| assignment.find_node(byte_index))
+                })
                 .or_else(|| self.self_if_contains_location(byte_index)),
         }
     }
@@ -388,12 +414,10 @@ impl TypedExpr {
         match self {
             Self::Var { .. }
             | Self::Int { .. }
-            | Self::Panic { .. }
             | Self::Float { .. }
             | Self::String { .. }
             | Self::ModuleSelect { .. }
-            | Self::Invalid { .. }
-            | Self::Todo { .. } => None,
+            | Self::Invalid { .. } => None,
 
             Self::Pipeline {
                 first_value,
@@ -474,9 +498,9 @@ impl TypedExpr {
 
             Self::Fn { body, .. } => body.iter().find_map(|s| s.find_statement(byte_index)),
 
-            Self::Call { fun, args, .. } => args
+            Self::Call { fun, arguments, .. } => arguments
                 .iter()
-                .find_map(|arg| arg.find_statement(byte_index))
+                .find_map(|argument| argument.find_statement(byte_index))
                 .or_else(|| fun.find_statement(byte_index)),
 
             Self::BinOp { left, right, .. } => left
@@ -501,19 +525,49 @@ impl TypedExpr {
                 tuple: expression, ..
             } => expression.find_statement(byte_index),
 
-            Self::Echo { expression, .. } => expression
+            Self::Echo {
+                expression,
+                message,
+                ..
+            } => expression
                 .as_ref()
-                .and_then(|expression| expression.find_statement(byte_index)),
+                .and_then(|expression| expression.find_statement(byte_index))
+                .or_else(|| {
+                    message
+                        .as_ref()
+                        .and_then(|message| message.find_statement(byte_index))
+                }),
+
+            Self::Todo { message, kind, .. } => match kind {
+                TodoKind::EmptyFunction { .. } | TodoKind::IncompleteUse | TodoKind::EmptyBlock => {
+                    None
+                }
+                TodoKind::Keyword => message
+                    .as_ref()
+                    .and_then(|message| message.find_statement(byte_index)),
+            },
+
+            Self::Panic { message, .. } => message
+                .as_ref()
+                .and_then(|message| message.find_statement(byte_index)),
 
             Self::BitArray { segments, .. } => segments
                 .iter()
                 .find_map(|arg| arg.value.find_statement(byte_index)),
 
-            Self::RecordUpdate { record, args, .. } => args
+            Self::RecordUpdate {
+                record_assignment,
+                arguments,
+                ..
+            } => arguments
                 .iter()
                 .filter(|arg| arg.implicit.is_none())
                 .find_map(|arg| arg.find_statement(byte_index))
-                .or_else(|| record.value.find_statement(byte_index)),
+                .or_else(|| {
+                    record_assignment
+                        .as_ref()
+                        .and_then(|r| r.value.find_statement(byte_index))
+                }),
         }
     }
 
@@ -526,14 +580,19 @@ impl TypedExpr {
     }
 
     pub fn non_zero_compile_time_number(&self) -> bool {
-        use regex::Regex;
-        static NON_ZERO: OnceLock<Regex> = OnceLock::new();
+        match self {
+            Self::Int { int_value, .. } => int_value != &BigInt::ZERO,
+            Self::Float { value, .. } => is_non_zero_number(value),
+            _ => false,
+        }
+    }
 
-        matches!(
-            self,
-            Self::Int{ value, .. } | Self::Float { value, .. } if NON_ZERO.get_or_init(||
-                Regex::new(r"[1-9]").expect("NON_ZERO regex")).is_match(value)
-        )
+    pub fn zero_compile_time_number(&self) -> bool {
+        match self {
+            Self::Int { int_value, .. } => int_value == &BigInt::ZERO,
+            Self::Float { value, .. } => !is_non_zero_number(value),
+            _ => false,
+        }
     }
 
     pub fn location(&self) -> SrcSpan {
@@ -675,9 +734,9 @@ impl TypedExpr {
             }
 
             // Calls are literals if they are records and all the arguemnts are also literals.
-            Self::Call { fun, args, .. } => {
-                fun.is_record_constructor()
-                    && args.iter().all(|argument| argument.value.is_literal())
+            Self::Call { fun, arguments, .. } => {
+                fun.is_record_builder()
+                    && arguments.iter().all(|argument| argument.value.is_literal())
             }
 
             // Variables are literals if they are record constructors that take no arguments.
@@ -694,14 +753,12 @@ impl TypedExpr {
         }
     }
 
-    pub fn is_known_value(&self) -> bool {
+    pub fn is_known_bool(&self) -> bool {
         match self {
-            TypedExpr::BinOp { left, right, .. } => left.is_known_value() && right.is_known_value(),
-
-            TypedExpr::NegateBool { value, .. } | TypedExpr::NegateInt { value, .. } => {
-                value.is_known_value()
-            }
-
+            TypedExpr::BinOp {
+                left, right, name, ..
+            } if name.is_bool_operator() => left.is_known_bool() && right.is_known_bool(),
+            TypedExpr::NegateBool { value, .. } => value.is_known_bool(),
             _ => self.is_literal(),
         }
     }
@@ -816,9 +873,9 @@ impl TypedExpr {
                     && finally.is_pure_value_constructor()
             }
 
-            TypedExpr::Call { fun, args, .. } => {
+            TypedExpr::Call { fun, arguments, .. } => {
                 (fun.is_record_builder() || fun.called_function_purity().is_pure())
-                    && args
+                    && arguments
                         .iter()
                         .all(|argument| argument.value.is_pure_value_constructor())
             }
@@ -915,21 +972,9 @@ impl TypedExpr {
     }
 
     #[must_use]
-    pub fn is_record_constructor(&self) -> bool {
-        match self {
-            TypedExpr::Var {
-                constructor:
-                    ValueConstructor {
-                        variant: ValueConstructorVariant::Record { .. },
-                        ..
-                    },
-                ..
-            } => true,
-            _ => false,
-        }
-    }
-
-    #[must_use]
+    /// Returns true if the value is a literal record builder like
+    /// `Wibble(1, 2)`, `module.Wobble("a")`
+    ///
     pub fn is_record_builder(&self) -> bool {
         match self {
             TypedExpr::Call { fun, .. } => fun.is_record_builder(),
@@ -942,6 +987,28 @@ impl TypedExpr {
         }
     }
 
+    /// If the given expression is a literal record builder, this will return
+    /// index of the variant being built.
+    ///
+    pub fn variant_index(&self) -> Option<u16> {
+        match self {
+            TypedExpr::Call { fun, .. } => fun.variant_index(),
+            TypedExpr::Var {
+                constructor:
+                    ValueConstructor {
+                        variant: ValueConstructorVariant::Record { variant_index, .. },
+                        ..
+                    },
+                ..
+            }
+            | TypedExpr::ModuleSelect {
+                constructor: ModuleValueConstructor::Record { variant_index, .. },
+                ..
+            } => Some(*variant_index),
+            _ => None,
+        }
+    }
+
     #[must_use]
     /// If `self` is a record constructor, returns the nuber of arguments it
     /// needs to be called. Otherwise, returns `None`.
@@ -971,7 +1038,7 @@ impl TypedExpr {
 
     pub(crate) fn call_arguments(&self) -> Option<&Vec<TypedCallArg>> {
         match self {
-            TypedExpr::Call { args, .. } => Some(args),
+            TypedExpr::Call { arguments, .. } => Some(arguments),
             _ => None,
         }
     }
@@ -1055,6 +1122,15 @@ impl TypedExpr {
     }
 }
 
+fn is_non_zero_number(value: &EcoString) -> bool {
+    use regex::Regex;
+    static NON_ZERO: OnceLock<Regex> = OnceLock::new();
+
+    NON_ZERO
+        .get_or_init(|| Regex::new(r"[1-9]").expect("NON_ZERO regex"))
+        .is_match(value)
+}
+
 impl<'a> From<&'a TypedExpr> for Located<'a> {
     fn from(expression: &'a TypedExpr) -> Self {
         Located::Expression {
