@@ -40,6 +40,7 @@
 
 use crate::{
     analyse::Inferred,
+    ast::{BitArraySize, TypedBitArraySize},
     exhaustiveness::CompiledCase,
     type_::{
         ModuleValueConstructor, PatternConstructor, TypedCallArg, ValueConstructor,
@@ -92,8 +93,9 @@ pub trait Visit<'ast> {
         location: &'ast SrcSpan,
         type_: &'ast Arc<Type>,
         expression: &'ast Option<Box<TypedExpr>>,
+        message: &'ast Option<Box<TypedExpr>>,
     ) {
-        visit_typed_expr_echo(self, location, type_, expression);
+        visit_typed_expr_echo(self, location, type_, expression, message);
     }
 
     fn visit_typed_expr_int(
@@ -163,11 +165,19 @@ pub trait Visit<'ast> {
         location: &'ast SrcSpan,
         type_: &'ast Arc<Type>,
         kind: &'ast FunctionLiteralKind,
-        args: &'ast [TypedArg],
+        arguments: &'ast [TypedArg],
         body: &'ast Vec1<TypedStatement>,
         return_annotation: &'ast Option<TypeAst>,
     ) {
-        visit_typed_expr_fn(self, location, type_, kind, args, body, return_annotation);
+        visit_typed_expr_fn(
+            self,
+            location,
+            type_,
+            kind,
+            arguments,
+            body,
+            return_annotation,
+        );
     }
 
     fn visit_typed_expr_list(
@@ -185,9 +195,9 @@ pub trait Visit<'ast> {
         location: &'ast SrcSpan,
         type_: &'ast Arc<Type>,
         fun: &'ast TypedExpr,
-        args: &'ast [TypedCallArg],
+        arguments: &'ast [TypedCallArg],
     ) {
-        visit_typed_expr_call(self, location, type_, fun, args);
+        visit_typed_expr_call(self, location, type_, fun, arguments);
     }
 
     fn visit_typed_expr_bin_op(
@@ -299,11 +309,11 @@ pub trait Visit<'ast> {
         &mut self,
         location: &'ast SrcSpan,
         type_: &'ast Arc<Type>,
-        record: &'ast TypedAssignment,
+        record: &'ast Option<Box<TypedAssignment>>,
         constructor: &'ast TypedExpr,
-        args: &'ast [TypedCallArg],
+        arguments: &'ast [TypedCallArg],
     ) {
-        visit_typed_expr_record_update(self, location, type_, record, constructor, args);
+        visit_typed_expr_record_update(self, location, type_, record, constructor, arguments);
     }
 
     fn visit_typed_expr_negate_bool(&mut self, location: &'ast SrcSpan, value: &'ast TypedExpr) {
@@ -435,14 +445,22 @@ pub trait Visit<'ast> {
         visit_typed_pattern_variable(self, location, name, type_, origin);
     }
 
-    fn visit_typed_pattern_var_usage(
+    fn visit_typed_pattern_bit_array_size(&mut self, size: &'ast TypedBitArraySize) {
+        visit_typed_pattern_bit_array_size(self, size);
+    }
+
+    fn visit_typed_bit_array_size_int(&mut self, location: &'ast SrcSpan, value: &'ast EcoString) {
+        visit_typed_bit_array_size_int(self, location, value)
+    }
+
+    fn visit_typed_bit_array_size_variable(
         &mut self,
         location: &'ast SrcSpan,
         name: &'ast EcoString,
-        constructor: &'ast Option<ValueConstructor>,
+        constructor: &'ast Option<Box<ValueConstructor>>,
         type_: &'ast Arc<Type>,
     ) {
-        visit_typed_pattern_var_usage(self, location, name, constructor, type_);
+        visit_typed_bit_array_size_variable(self, location, name, constructor, type_)
     }
 
     fn visit_typed_pattern_assign(
@@ -587,8 +605,8 @@ pub fn visit_typed_module<'a, V>(v: &mut V, module: &'a TypedModule)
 where
     V: Visit<'a> + ?Sized,
 {
-    for def in &module.definitions {
-        v.visit_typed_definition(def);
+    for definition in &module.definitions {
+        v.visit_typed_definition(definition);
     }
 }
 
@@ -704,10 +722,15 @@ where
 {
 }
 
-pub fn visit_typed_custom_type<'a, V>(_v: &mut V, _custom_type: &'a TypedCustomType)
+pub fn visit_typed_custom_type<'a, V>(v: &mut V, custom_type: &'a TypedCustomType)
 where
     V: Visit<'a> + ?Sized,
 {
+    for record in &custom_type.constructors {
+        for argument in &record.arguments {
+            v.visit_type_ast(&argument.ast);
+        }
+    }
 }
 
 pub fn visit_typed_expr<'a, V>(v: &mut V, node: &'a TypedExpr)
@@ -751,11 +774,11 @@ where
             location,
             type_,
             kind,
-            args,
+            arguments,
             body,
             return_annotation,
             purity: _,
-        } => v.visit_typed_expr_fn(location, type_, kind, args, body, return_annotation),
+        } => v.visit_typed_expr_fn(location, type_, kind, arguments, body, return_annotation),
         TypedExpr::List {
             location,
             type_,
@@ -766,8 +789,8 @@ where
             location,
             type_,
             fun,
-            args,
-        } => v.visit_typed_expr_call(location, type_, fun, args),
+            arguments,
+        } => v.visit_typed_expr_call(location, type_, fun, arguments),
         TypedExpr::BinOp {
             location,
             type_,
@@ -838,10 +861,16 @@ where
         TypedExpr::RecordUpdate {
             location,
             type_,
-            record,
+            record_assignment,
             constructor,
-            args,
-        } => v.visit_typed_expr_record_update(location, type_, record, constructor, args),
+            arguments,
+        } => v.visit_typed_expr_record_update(
+            location,
+            type_,
+            record_assignment,
+            constructor,
+            arguments,
+        ),
         TypedExpr::NegateBool { location, value } => {
             v.visit_typed_expr_negate_bool(location, value)
         }
@@ -850,8 +879,9 @@ where
         TypedExpr::Echo {
             location,
             expression,
+            message,
             type_,
-        } => v.visit_typed_expr_echo(location, type_, expression),
+        } => v.visit_typed_expr_echo(location, type_, expression, message),
     }
 }
 
@@ -938,7 +968,7 @@ pub fn visit_typed_expr_fn<'a, V>(
     _location: &'a SrcSpan,
     _type_: &'a Arc<Type>,
     _kind: &'a FunctionLiteralKind,
-    _args: &'a [TypedArg],
+    _arguments: &'a [TypedArg],
     body: &'a Vec1<TypedStatement>,
     _return_annotation: &'a Option<TypeAst>,
 ) where
@@ -972,13 +1002,13 @@ pub fn visit_typed_expr_call<'a, V>(
     _location: &'a SrcSpan,
     _type_: &'a Arc<Type>,
     fun: &'a TypedExpr,
-    args: &'a [TypedCallArg],
+    arguments: &'a [TypedCallArg],
 ) where
     V: Visit<'a> + ?Sized,
 {
     v.visit_typed_expr(fun);
-    for arg in args {
-        v.visit_typed_call_arg(arg);
+    for argument in arguments {
+        v.visit_typed_call_arg(argument);
     }
 }
 
@@ -1090,19 +1120,23 @@ pub fn visit_typed_expr_echo<'a, V>(
     _location: &'a SrcSpan,
     _type_: &'a Arc<Type>,
     expression: &'a Option<Box<TypedExpr>>,
+    message: &'a Option<Box<TypedExpr>>,
 ) where
     V: Visit<'a> + ?Sized,
 {
     if let Some(expression) = expression {
         v.visit_typed_expr(expression)
     }
+    if let Some(message) = message {
+        v.visit_typed_expr(message)
+    }
 }
 
 pub fn visit_typed_expr_panic<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
     message: &'a Option<Box<TypedExpr>>,
-    _type: &'a Arc<Type>,
+    _type_: &'a Arc<Type>,
 ) where
     V: Visit<'a> + ?Sized,
 {
@@ -1127,17 +1161,19 @@ pub fn visit_typed_expr_bit_array<'a, V>(
 pub fn visit_typed_expr_record_update<'a, V>(
     v: &mut V,
     _location: &'a SrcSpan,
-    _type: &'a Arc<Type>,
-    record: &'a TypedAssignment,
+    _type_: &'a Arc<Type>,
+    record: &'a Option<Box<TypedAssignment>>,
     constructor: &'a TypedExpr,
-    args: &'a [TypedCallArg],
+    arguments: &'a [TypedCallArg],
 ) where
     V: Visit<'a> + ?Sized,
 {
     v.visit_typed_expr(constructor);
-    v.visit_typed_assignment(record);
-    for arg in args {
-        v.visit_typed_call_arg(arg);
+    if let Some(record) = record {
+        v.visit_typed_assignment(record);
+    }
+    for argument in arguments {
+        v.visit_typed_call_arg(argument);
     }
 }
 
@@ -1484,12 +1520,7 @@ where
             type_,
             origin,
         } => v.visit_typed_pattern_variable(location, name, type_, origin),
-        Pattern::VarUsage {
-            location,
-            name,
-            constructor,
-            type_,
-        } => v.visit_typed_pattern_var_usage(location, name, constructor, type_),
+        Pattern::BitArraySize(size) => v.visit_typed_pattern_bit_array_size(size),
         Pattern::Assign {
             location,
             name,
@@ -1570,19 +1601,52 @@ pub fn visit_typed_pattern_variable<'a, V>(
     _v: &mut V,
     _location: &'a SrcSpan,
     _name: &'a EcoString,
-    _type: &'a Arc<Type>,
+    _type_: &'a Arc<Type>,
     _origin: &'a VariableOrigin,
 ) where
     V: Visit<'a> + ?Sized,
 {
 }
 
-pub fn visit_typed_pattern_var_usage<'a, V>(
+pub fn visit_typed_pattern_bit_array_size<'a, V>(v: &mut V, size: &'a TypedBitArraySize)
+where
+    V: Visit<'a> + ?Sized,
+{
+    match size {
+        BitArraySize::Int {
+            location,
+            value,
+            int_value: _,
+        } => v.visit_typed_bit_array_size_int(location, value),
+        BitArraySize::Variable {
+            location,
+            name,
+            constructor,
+            type_,
+        } => v.visit_typed_bit_array_size_variable(location, name, constructor, type_),
+        BitArraySize::BinaryOperator { left, right, .. } => {
+            v.visit_typed_pattern_bit_array_size(left);
+            v.visit_typed_pattern_bit_array_size(right);
+        }
+        BitArraySize::Block { inner, .. } => v.visit_typed_pattern_bit_array_size(inner),
+    }
+}
+
+pub fn visit_typed_bit_array_size_int<'a, V>(
+    _v: &mut V,
+    _location: &'a SrcSpan,
+    _value: &'a EcoString,
+) where
+    V: Visit<'a> + ?Sized,
+{
+}
+
+pub fn visit_typed_bit_array_size_variable<'a, V>(
     _v: &mut V,
     _location: &'a SrcSpan,
     _name: &'a EcoString,
-    _constructor: &'a Option<ValueConstructor>,
-    _type: &'a Arc<Type>,
+    _constructor: &'a Option<Box<ValueConstructor>>,
+    _type_: &'a Arc<Type>,
 ) where
     V: Visit<'a> + ?Sized,
 {
@@ -1603,7 +1667,7 @@ pub fn visit_typed_pattern_discard<'a, V>(
     _v: &mut V,
     _location: &'a SrcSpan,
     _name: &'a EcoString,
-    _type: &'a Arc<Type>,
+    _type_: &'a Arc<Type>,
 ) where
     V: Visit<'a> + ?Sized,
 {
@@ -1614,7 +1678,7 @@ pub fn visit_typed_pattern_list<'a, V>(
     _location: &'a SrcSpan,
     elements: &'a Vec<TypedPattern>,
     tail: &'a Option<Box<TypedPattern>>,
-    _type: &'a Arc<Type>,
+    _type_: &'a Arc<Type>,
 ) where
     V: Visit<'a> + ?Sized,
 {
@@ -1636,7 +1700,7 @@ pub fn visit_typed_pattern_constructor<'a, V>(
     _module: &'a Option<(EcoString, SrcSpan)>,
     _constructor: &'a Inferred<PatternConstructor>,
     _spread: &'a Option<SrcSpan>,
-    _type: &'a Arc<Type>,
+    _type_: &'a Arc<Type>,
 ) where
     V: Visit<'a> + ?Sized,
 {
