@@ -2,6 +2,7 @@ use ecow::EcoString;
 use num_bigint::BigInt;
 
 use crate::ast::{self, BitArrayOption, SrcSpan};
+use crate::build::Target;
 use crate::type_::Type;
 use std::sync::Arc;
 
@@ -11,21 +12,28 @@ use std::sync::Arc;
 
 pub fn type_options_for_value<TypedValue>(
     input_options: &[BitArrayOption<TypedValue>],
+    target: Target,
 ) -> Result<Arc<Type>, Error>
 where
     TypedValue: GetLiteralValue,
 {
-    type_options(input_options, TypeOptionsMode::Expression, false)
+    type_options(input_options, TypeOptionsMode::Expression, false, target)
 }
 
 pub fn type_options_for_pattern<TypedValue>(
     input_options: &[BitArrayOption<TypedValue>],
     must_have_size: bool,
+    target: Target,
 ) -> Result<Arc<Type>, Error>
 where
     TypedValue: GetLiteralValue,
 {
-    type_options(input_options, TypeOptionsMode::Pattern, must_have_size)
+    type_options(
+        input_options,
+        TypeOptionsMode::Pattern,
+        must_have_size,
+        target,
+    )
 }
 
 struct SegmentOptionCategories<'a, T> {
@@ -86,6 +94,7 @@ fn type_options<TypedValue>(
     input_options: &[BitArrayOption<TypedValue>],
     mode: TypeOptionsMode,
     must_have_size: bool,
+    target: Target,
 ) -> Result<Arc<Type>, Error>
 where
     TypedValue: GetLiteralValue,
@@ -96,6 +105,18 @@ where
     // Basic category checking
     for option in input_options {
         match option {
+            Utf8Codepoint { .. } | Utf16Codepoint { .. } | Utf32Codepoint { .. }
+                if mode == TypeOptionsMode::Pattern && target == Target::JavaScript =>
+            {
+                return err(
+                    ErrorType::OptionNotSupportedForTarget {
+                        target,
+                        option: UnsupportedOption::UtfCodepointPattern,
+                    },
+                    option.location(),
+                );
+            }
+
             Bytes { .. }
             | Int { .. }
             | Float { .. }
@@ -131,6 +152,16 @@ where
                 }
             }
 
+            Native { .. } if target == Target::JavaScript => {
+                return err(
+                    ErrorType::OptionNotSupportedForTarget {
+                        target,
+                        option: UnsupportedOption::NativeEndianness,
+                    },
+                    option.location(),
+                );
+            }
+
             Big { .. } | Little { .. } | Native { .. } => {
                 if let Some(previous) = categories.endian {
                     return err(
@@ -188,10 +219,20 @@ where
         }
     }
 
-    // Endianness is only valid for int, utf16, utf32 and float
+    // Endianness is only valid for int, utf16, utf16_codepoint, utf32,
+    // utf32_codepoint and float
     match categories {
         SegmentOptionCategories {
-            type_: None | Some(Int { .. } | Utf16 { .. } | Utf32 { .. } | Float { .. }),
+            type_:
+                None
+                | Some(
+                    Int { .. }
+                    | Utf16 { .. }
+                    | Utf32 { .. }
+                    | Utf16Codepoint { .. }
+                    | Utf32Codepoint { .. }
+                    | Float { .. },
+                ),
             ..
         } => {}
 
@@ -303,10 +344,12 @@ pub trait GetLiteralValue {
 
 impl GetLiteralValue for ast::TypedPattern {
     fn as_int_literal(&self) -> Option<BigInt> {
-        if let ast::Pattern::Int { int_value, .. } = self {
-            Some(int_value.clone())
-        } else {
-            None
+        match self {
+            ast::Pattern::Int { int_value, .. }
+            | ast::Pattern::BitArraySize(ast::BitArraySize::Int { int_value, .. }) => {
+                Some(int_value.clone())
+            }
+            _ => None,
         }
     }
 }
@@ -337,19 +380,41 @@ pub struct Error {
 
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub enum ErrorType {
-    ConflictingEndiannessOptions { existing_endianness: EcoString },
-    ConflictingSignednessOptions { existing_signed: EcoString },
+    ConflictingEndiannessOptions {
+        existing_endianness: EcoString,
+    },
+    ConflictingSignednessOptions {
+        existing_signed: EcoString,
+    },
     ConflictingSizeOptions,
-    ConflictingTypeOptions { existing_type: EcoString },
+    ConflictingTypeOptions {
+        existing_type: EcoString,
+    },
     ConflictingUnitOptions,
     FloatWithSize,
     InvalidEndianness,
     OptionNotAllowedInValue,
     SegmentMustHaveSize,
-    SignednessUsedOnNonInt { type_: EcoString },
-    TypeDoesNotAllowSize { type_: EcoString },
-    TypeDoesNotAllowUnit { type_: EcoString },
+    SignednessUsedOnNonInt {
+        type_: EcoString,
+    },
+    TypeDoesNotAllowSize {
+        type_: EcoString,
+    },
+    TypeDoesNotAllowUnit {
+        type_: EcoString,
+    },
     UnitMustHaveSize,
     VariableUtfSegmentInPattern,
     ConstantSizeNotPositive,
+    OptionNotSupportedForTarget {
+        target: Target,
+        option: UnsupportedOption,
+    },
+}
+
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+pub enum UnsupportedOption {
+    UtfCodepointPattern,
+    NativeEndianness,
 }
