@@ -335,11 +335,14 @@ impl Printer<'_> {
         }
         self.register_local_type_variable_names(return_type);
 
-        let arguments = arguments.iter().map(|argument| {
-            let name = self.variable(self.argument_name(argument));
-            docvec![name, ": ", self.type_(&argument.type_)].group()
-        });
-        let arguments = Self::wrap_arguments(arguments);
+        let arguments = if arguments.is_empty() {
+            "()".to_doc()
+        } else {
+            Self::wrap_arguments(arguments.iter().map(|argument| {
+                let name = self.variable(self.argument_name(argument));
+                docvec![name, ": ", self.type_(&argument.type_)].group()
+            }))
+        };
 
         docvec![
             self.keyword("pub fn "),
@@ -390,22 +393,22 @@ impl Printer<'_> {
                 package,
                 module,
                 name,
-                args,
+                arguments,
                 publicity,
                 ..
             } => {
                 let name = self.named_type_name(publicity, package, module, name);
-                if args.is_empty() {
+                if arguments.is_empty() {
                     name
                 } else {
                     name.append(Self::type_arguments(
-                        args.iter().map(|argument| self.type_(argument)),
+                        arguments.iter().map(|argument| self.type_(argument)),
                     ))
                 }
             }
-            Type::Fn { args, return_ } => docvec![
+            Type::Fn { arguments, return_ } => docvec![
                 self.keyword("fn"),
-                Self::type_arguments(args.iter().map(|argument| self.type_(argument))),
+                Self::type_arguments(arguments.iter().map(|argument| self.type_(argument))),
                 " -> ",
                 self.type_(return_)
             ],
@@ -582,14 +585,14 @@ impl Printer<'_> {
     ///
     fn register_local_type_variable_names(&mut self, type_: &Type) {
         match type_ {
-            Type::Named { args, .. } => {
-                for arg in args {
-                    self.register_local_type_variable_names(arg);
+            Type::Named { arguments, .. } => {
+                for argument in arguments {
+                    self.register_local_type_variable_names(argument);
                 }
             }
-            Type::Fn { args, return_ } => {
-                for arg in args {
-                    self.register_local_type_variable_names(arg);
+            Type::Fn { arguments, return_ } => {
+                for argument in arguments {
+                    self.register_local_type_variable_names(argument);
                 }
                 self.register_local_type_variable_names(return_);
             }
