@@ -26,6 +26,7 @@ use crate::{
 use camino::Utf8PathBuf;
 use ecow::EcoString;
 use hexpm::version::Version;
+use http::Uri;
 use itertools::Itertools;
 use serde_json::to_string as serde_to_string;
 
@@ -192,12 +193,12 @@ fn compile_documentation(
         .definitions
         .iter()
         .filter_map(
-            |statement: &crate::ast::Definition<
+            |definition: &crate::ast::Definition<
                 std::sync::Arc<type_::Type>,
                 crate::ast::TypedExpr,
                 EcoString,
                 EcoString,
-            >| printer.type_definition(&source_links, statement),
+            >| printer.type_definition(&source_links, definition),
         )
         .sorted()
         .collect_vec();
@@ -205,7 +206,7 @@ fn compile_documentation(
     let values = module
         .definitions
         .iter()
-        .filter_map(|statement| printer.value(&source_links, statement))
+        .filter_map(|definition| printer.value(&source_links, definition))
         .sorted()
         .collect_vec();
 
@@ -617,32 +618,34 @@ pub type Wibble = Int
 fn source_link_for_github_repository() {
     let mut config = PackageConfig::default();
     config.name = EcoString::from("test_project_name");
-    config.repository = Repository::GitHub {
+    config.repository = Some(Repository::GitHub {
         user: "wibble".to_string(),
         repo: "wobble".to_string(),
         path: None,
-    };
+        tag_prefix: None,
+    });
 
     let modules = vec![("app.gleam", "pub type Wibble = Int")];
     assert!(
         compile(config, modules)
-            .contains("https://github.com/wibble/wobble/blob/v0.1.0/src/app.gleam#L1-L1")
+            .contains("https://github.com/wibble/wobble/blob/v0.1.0/src/app.gleam#L1")
     );
 }
 
 #[test]
-fn source_link_for_github_repository_with_path() {
+fn source_link_for_github_repository_with_path_and_tag_prefix() {
     let mut config = PackageConfig::default();
     config.name = EcoString::from("test_project_name");
-    config.repository = Repository::GitHub {
+    config.repository = Some(Repository::GitHub {
         user: "wibble".to_string(),
         repo: "wobble".to_string(),
         path: Some("path/to/package".to_string()),
-    };
+        tag_prefix: Some("subdir-".into()),
+    });
 
     let modules = vec![("app.gleam", "pub type Wibble = Int")];
     assert!(compile(config, modules).contains(
-        "https://github.com/wibble/wobble/blob/v0.1.0/path/to/package/src/app.gleam#L1-L1"
+        "https://github.com/wibble/wobble/blob/subdir-v0.1.0/path/to/package/src/app.gleam#L1"
     ));
 }
 
@@ -1162,3 +1165,28 @@ pub type Wibble {
         NONE
     );
 }
+
+#[test]
+fn gitea_repository_url_has_no_double_slash() {
+    let repo = Repository::Forgejo {
+        host: "https://code.example.org/".parse::<Uri>().unwrap(),
+        user: "person".into(),
+        repo: "forgejo_bug".into(),
+        path: None,
+        tag_prefix: None,
+    };
+
+    assert_eq!(repo.url(), "https://code.example.org/person/forgejo_bug");
+}
+
+#[test]
+fn long_function_with_no_arguments_parentheses_are_not_split() {
+    assert_documentation!(
+        "
+pub fn aaaaaaaaaaaaaaaaaaaaaaaaaaaa() -> aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa {
+  todo
+}
+",
+        NONE
+    );
+}
