@@ -31,7 +31,7 @@ fn bit_array_float() {
 fn bit_array1() {
     assert_erl!(
         r#"pub fn x() { 2 }
-fn main() {
+pub fn main() {
   let a = -1
   let b = <<a:unit(2)-size(a * 2), a:size(3 + x())-unit(1)>>
 
@@ -209,3 +209,108 @@ fn identity(x) {
 "
     );
 }
+
+#[test]
+fn utf16_codepoint_little_endian() {
+    assert_erl!(
+        "
+pub fn go(codepoint) {
+  <<codepoint:utf16_codepoint-little>>
+}
+"
+    );
+}
+
+#[test]
+fn utf32_codepoint_little_endian() {
+    assert_erl!(
+        "
+pub fn go(codepoint) {
+  <<codepoint:utf32_codepoint-little>>
+}
+"
+    );
+}
+
+#[test]
+fn pattern_match_utf16_codepoint_little_endian() {
+    assert_erl!(
+        "
+pub fn go(x) {
+  let assert <<codepoint:utf16_codepoint-little>> = x
+  codepoint
+}
+"
+    );
+}
+
+#[test]
+fn pattern_match_utf32_codepoint_little_endian() {
+    assert_erl!(
+        "
+pub fn go(x) {
+  let assert <<codepoint:utf32_codepoint-little>> = x
+  codepoint
+}
+"
+    );
+}
+
+#[test]
+fn operator_in_pattern_size() {
+    assert_erl!(
+        "
+pub fn main() {
+  let assert <<len, payload:size(len * 8 - 8)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn operator_in_pattern_size2() {
+    assert_erl!(
+        "
+pub fn main() {
+  let assert <<len, payload:size(len / 8 - 1)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn operator_in_pattern_size3() {
+    assert_erl!(
+        "
+pub fn main() {
+  let additional = 10
+  let assert <<len, payload:size(len + additional * 8)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn block_in_pattern_size() {
+    assert_erl!(
+        "
+pub fn main() {
+  let assert <<len, payload:size({ len - 1 } * 8)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn non_byte_aligned_size_calculation() {
+    assert_erl!(
+        "
+pub fn main() {
+  case <<>> {
+    <<a:1, b:3, c:size(b - 2)>> -> c + b
+    _ -> 1
+  }
+}
+"
+    );
+}
