@@ -7,11 +7,13 @@ mod blocks;
 mod cases;
 mod conditional_compilation;
 mod custom_type;
+mod echo;
 mod external_fn;
 mod external_types;
 mod function;
 mod guards;
 mod imports;
+mod lists;
 mod pipeline;
 mod record_update;
 mod tuple;
@@ -5992,9 +5994,11 @@ fn const_concat_long_including_list() {
         r#"const x = "some long string 1"
   <> "some long string 2"
   <> [
-    "here is a list", "with several elements",
+    "here is a list",
+    "with several elements",
     "in order to make it be too long to fit on one line",
-    "so we can see how it breaks", "onto multiple lines",
+    "so we can see how it breaks",
+    "onto multiple lines",
   ]
   <> "and a last string"
 "#,
@@ -6401,146 +6405,6 @@ fn function_capture_formatted_like_regular_calls_in_a_pipe() {
     );
 }
 
-#[test]
-fn echo() {
-    assert_format!(
-        "fn main() {
-  echo
-}
-"
-    );
-}
-
-#[test]
-fn echo_with_value() {
-    assert_format!(
-        r#"fn main() {
-  echo value
-}
-"#
-    );
-}
-
-#[test]
-fn echo_with_big_value_that_needs_to_be_split() {
-    assert_format!(
-        r#"fn main() {
-  echo [
-    this_is_a_long_list_and_requires_splitting,
-    wibble_wobble_woo,
-    multiple_lines,
-  ]
-}
-"#
-    );
-}
-
-#[test]
-fn echo_inside_a_pipeline() {
-    assert_format!(
-        r#"fn main() {
-  wibble
-  |> echo
-  |> wobble
-}
-"#
-    );
-}
-
-#[test]
-fn echo_inside_a_single_line_pipeline() {
-    assert_format!(
-        r#"fn main() {
-  wibble |> echo |> wobble
-}
-"#
-    );
-}
-
-#[test]
-fn echo_as_last_item_of_pipeline() {
-    assert_format!(
-        r#"fn main() {
-  wibble |> wobble |> echo
-}
-"#
-    );
-}
-
-#[test]
-fn echo_as_last_item_of_multiline_pipeline() {
-    assert_format!(
-        r#"fn main() {
-  wibble
-  |> wobble
-  |> echo
-}
-"#
-    );
-}
-
-#[test]
-fn echo_with_related_expression_on_following_line() {
-    assert_format_rewrite!(
-        r#"fn main() {
-  panic as echo
-  "wibble"
-}
-"#,
-        r#"fn main() {
-  panic as echo "wibble"
-}
-"#
-    );
-}
-
-#[test]
-fn echo_with_following_value_in_a_pipeline() {
-    assert_format_rewrite!(
-        r#"fn main() {
-  []
-  |> echo wibble
-  |> wobble
-}
-"#,
-        r#"fn main() {
-  []
-  |> echo
-  wibble
-  |> wobble
-}
-"#
-    );
-}
-
-#[test]
-fn echo_printing_multiline_pipeline() {
-    assert_format_rewrite!(
-        r#"fn main() {
-  echo first
-  |> wibble
-  |> wobble
-}
-"#,
-        r#"fn main() {
-  echo first
-    |> wibble
-    |> wobble
-}
-"#
-    );
-}
-
-#[test]
-fn echo_printing_one_line_pipeline() {
-    assert_format!(
-        r#"fn main() {
-  echo first |> wibble |> wobble
-}
-"#
-    );
-}
-
 #[test]
 fn assert() {
     assert_format!(
@@ -6626,3 +6490,198 @@ fn assert_with_long_binary_expression() {
 "#
     );
 }
+
+#[test]
+fn comment_is_not_moved_after_assert() {
+    assert_format!(
+        "pub fn main() {
+  // Wibble!
+  assert True
+}
+"
+    );
+}
+
+#[test]
+fn todo_as_with_comment() {
+    assert_format!(
+        r#"pub fn main() {
+  todo as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn todo_as_with_comment_on_the_same_line() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  todo as // A little comment explaining something
+    "wibble"
+}
+"#,
+        r#"pub fn main() {
+  todo as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn todo_as_with_comment_before_the_as() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  todo // A little comment explaining something
+    as "wibble"
+}
+"#,
+        r#"pub fn main() {
+  todo as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn panic_as_with_comment() {
+    assert_format!(
+        r#"pub fn main() {
+  panic as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn panic_as_with_comment_on_the_same_line() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  panic as // A little comment explaining something
+    "wibble"
+}
+"#,
+        r#"pub fn main() {
+  panic as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn panic_as_with_comment_before_the_as() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  panic // A little comment explaining something
+    as "wibble"
+}
+"#,
+        r#"pub fn main() {
+  panic as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn echo_as_with_comment() {
+    assert_format!(
+        r#"pub fn main() {
+  echo 1 as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn echo_as_with_comment_on_the_same_line() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  echo 1 as // A little comment explaining something
+    "wibble"
+}
+"#,
+        r#"pub fn main() {
+  echo 1 as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn echo_as_with_comment_before_the_as() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  echo 1 // A little comment explaining something
+    as "wibble"
+}
+"#,
+        r#"pub fn main() {
+  echo 1 as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn assert_as_with_comment() {
+    assert_format!(
+        r#"pub fn main() {
+  assert True as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn assert_as_with_comment_on_the_same_line() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  assert True as // A little comment explaining something
+    "wibble"
+}
+"#,
+        r#"pub fn main() {
+  assert True as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
+
+#[test]
+fn assert_as_with_comment_before_the_as() {
+    assert_format_rewrite!(
+        r#"pub fn main() {
+  assert True // A little comment explaining something
+    as "wibble"
+}
+"#,
+        r#"pub fn main() {
+  assert True as
+    // A little comment explaining something
+    "wibble"
+}
+"#
+    );
+}
