@@ -8,7 +8,6 @@ mod typescript;
 use num_bigint::BigInt;
 use num_traits::ToPrimitive;
 
-use crate::analyse::TargetSupport;
 use crate::build::Target;
 use crate::build::package_compiler::StdlibPackage;
 use crate::codegen::TypeScriptDeclarations;
@@ -31,8 +30,6 @@ const INDENT: isize = 2;
 pub const PRELUDE: &str = include_str!("../templates/prelude.mjs");
 pub const PRELUDE_TS_DEF: &str = include_str!("../templates/prelude.d.mts");
 
-pub type Output<'a> = Result<Document<'a>, Error>;
-
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum JavaScriptCodegenTarget {
     JavaScript,
@@ -46,7 +43,6 @@ pub struct Generator<'a> {
     tracker: UsageTracker,
     module_scope: im::HashMap<EcoString, usize>,
     current_module_name_segments_count: usize,
-    target_support: TargetSupport,
     typescript: TypeScriptDeclarations,
     stdlib_package: StdlibPackage,
     /// Relative path to the module, surrounded in `"`s to make it a string, and with `\`s escaped
@@ -57,7 +53,6 @@ pub struct Generator<'a> {
 impl<'a> Generator<'a> {
     pub fn new(config: ModuleConfig<'a>) -> Self {
         let ModuleConfig {
-            target_support,
             typescript,
             stdlib_package,
             module,
@@ -82,7 +77,6 @@ impl<'a> Generator<'a> {
             src_path,
             tracker: UsageTracker::default(),
             module_scope: Default::default(),
-            target_support,
             typescript,
             stdlib_package,
         }
@@ -105,7 +99,7 @@ impl<'a> Generator<'a> {
         docvec!["/// <reference types=\"./", module, ".d.mts\" />", line()]
     }
 
-    pub fn compile(&mut self) -> Output<'a> {
+    pub fn compile(&mut self) -> Document<'a> {
         // Determine what JavaScript imports we need to generate
         let mut imports = self.collect_imports();
 
@@ -119,12 +113,11 @@ impl<'a> Generator<'a> {
             self.module
                 .definitions
                 .iter()
-                .flat_map(|s| self.statement(s)),
+                .flat_map(|definition| self.definition(definition)),
         );
 
         // Two lines between each statement
-        let mut statements: Vec<_> =
-            Itertools::intersperse(statements, Ok(lines(2))).try_collect()?;
+        let mut statements = Itertools::intersperse(statements, lines(2)).collect_vec();
 
         // Import any prelude functions that have been used
 
@@ -140,11 +133,11 @@ impl<'a> Generator<'a> {
             self.register_prelude_usage(&mut imports, "toList", None);
         };
 
-        if self.tracker.list_empty_class_used {
+        if self.tracker.list_empty_class_used || self.tracker.echo_used {
             self.register_prelude_usage(&mut imports, "Empty", Some("$Empty"));
         };
 
-        if self.tracker.list_non_empty_class_used {
+        if self.tracker.list_non_empty_class_used || self.tracker.echo_used {
             self.register_prelude_usage(&mut imports, "NonEmpty", Some("$NonEmpty"));
         };
 
@@ -231,30 +224,30 @@ impl<'a> Generator<'a> {
         // Put it all together
 
         if imports.is_empty() && statements.is_empty() {
-            Ok(docvec![
+            docvec![
                 type_reference,
                 filepath_definition,
                 "export {}",
                 line(),
                 echo_definition
-            ])
+            ]
         } else if imports.is_empty() {
             statements.push(line());
-            Ok(docvec![
+            docvec![
                 type_reference,
                 filepath_definition,
                 statements,
                 echo_definition
-            ])
+            ]
         } else if statements.is_empty() {
-            Ok(docvec![
+            docvec![
                 type_reference,
                 imports.into_doc(JavaScriptCodegenTarget::JavaScript),
                 filepath_definition,
                 echo_definition,
-            ])
+            ]
         } else {
-            Ok(docvec![
+            docvec![
                 type_reference,
                 imports.into_doc(JavaScriptCodegenTarget::JavaScript),
                 line(),
@@ -262,7 +255,7 @@ impl<'a> Generator<'a> {
                 statements,
                 line(),
                 echo_definition
-            ])
+            ]
         }
     }
 
@@ -298,8 +291,8 @@ impl<'a> Generator<'a> {
         imports.register_module(path, [], [member]);
     }
 
-    pub fn statement(&mut self, statement: &'a TypedDefinition) -> Option<Output<'a>> {
-        match statement {
+    pub fn definition(&mut self, definition: &'a TypedDefinition) -> Option<Document<'a>> {
+        match definition {
             Definition::TypeAlias(TypeAlias { .. }) => None,
 
             // Handled in collect_imports
@@ -308,12 +301,24 @@ impl<'a> Generator<'a> {
             // Handled in collect_definitions
             Definition::CustomType(CustomType { .. }) => None,
 
+            // If a definition is unused then we don't need to generate code for it
+            Definition::ModuleConstant(ModuleConstant { location, .. })
+            | Definition::Function(Function { location, .. })
+                if self
+                    .module
+                    .unused_definition_positions
+                    .contains(&location.start) =>
+            {
+                None
+            }
+
             Definition::ModuleConstant(ModuleConstant {
                 publicity,
                 name,
                 value,
+                documentation,
                 ..
-            }) => Some(self.module_constant(*publicity, name, value)),
+            }) => Some(self.module_constant(*publicity, name, value, documentation)),
 
             Definition::Function(function) => {
                 // If there's an external JavaScript implementation then it will be imported,
@@ -328,7 +333,7 @@ impl<'a> Generator<'a> {
                     return None;
                 }
 
-                self.module_function(function)
+                Some(self.module_function(function))
             }
         }
     }
@@ -338,7 +343,7 @@ impl<'a> Generator<'a> {
         constructors: &'a [TypedRecordConstructor],
         publicity: Publicity,
         opaque: bool,
-    ) -> Vec<Output<'a>> {
+    ) -> Vec<Document<'a>> {
         // If there's no constructors then there's nothing to do here.
         if constructors.is_empty() {
             return vec![];
@@ -347,7 +352,7 @@ impl<'a> Generator<'a> {
         self.tracker.custom_type_used = true;
         constructors
             .iter()
-            .map(|constructor| Ok(self.record_definition(constructor, publicity, opaque)))
+            .map(|constructor| self.record_definition(constructor, publicity, opaque))
             .collect()
     }
 
@@ -365,6 +370,12 @@ impl<'a> Generator<'a> {
                 .to_doc()
         }
 
+        let doc = if let Some((_, documentation)) = &constructor.documentation {
+            jsdoc_comment(documentation, publicity).append(line())
+        } else {
+            nil()
+        };
+
         let head = if publicity.is_private() || opaque {
             "class "
         } else {
@@ -405,14 +416,24 @@ impl<'a> Generator<'a> {
         ]
         .nest(INDENT);
 
-        docvec![head, class_body, line(), "}"]
+        docvec![doc, head, class_body, line(), "}"]
     }
 
-    fn collect_definitions(&mut self) -> Vec<Output<'a>> {
+    fn collect_definitions(&mut self) -> Vec<Document<'a>> {
         self.module
             .definitions
             .iter()
-            .flat_map(|statement| match statement {
+            .flat_map(|definition| match definition {
+                // If a custom type is unused then we don't need to generate code for it
+                Definition::CustomType(CustomType { location, .. })
+                    if self
+                        .module
+                        .unused_definition_positions
+                        .contains(&location.start) =>
+                {
+                    vec![]
+                }
+
                 Definition::CustomType(CustomType {
                     publicity,
                     constructors,
@@ -431,8 +452,8 @@ impl<'a> Generator<'a> {
     fn collect_imports(&mut self) -> Imports<'a> {
         let mut imports = Imports::new();
 
-        for statement in &self.module.definitions {
-            match statement {
+        for definition in &self.module.definitions {
+            match definition {
                 Definition::Import(Import {
                     module,
                     as_name,
@@ -553,7 +574,8 @@ impl<'a> Generator<'a> {
         publicity: Publicity,
         name: &'a EcoString,
         value: &'a TypedConstant,
-    ) -> Output<'a> {
+        documentation: &'a Option<(u32, EcoString)>,
+    ) -> Document<'a> {
         let head = if publicity.is_private() {
             "const "
         } else {
@@ -570,22 +592,29 @@ impl<'a> Generator<'a> {
             self.module_scope.clone(),
         );
 
-        let document = generator.constant_expression(Context::Constant, value)?;
+        let document = generator.constant_expression(Context::Constant, value);
 
-        Ok(docvec![
+        let jsdoc = if let Some((_, documentation)) = documentation {
+            jsdoc_comment(documentation, publicity).append(line())
+        } else {
+            nil()
+        };
+
+        docvec![
+            jsdoc,
             head,
             maybe_escape_identifier(name),
             " = ",
             document,
             ";",
-        ])
+        ]
     }
 
     fn register_in_scope(&mut self, name: &str) {
         let _ = self.module_scope.insert(name.into(), 0);
     }
 
-    fn module_function(&mut self, function: &'a TypedFunction) -> Option<Output<'a>> {
+    fn module_function(&mut self, function: &'a TypedFunction) -> Document<'a> {
         let (_, name) = function
             .name
             .as_ref()
@@ -604,42 +633,37 @@ impl<'a> Generator<'a> {
             &mut self.tracker,
             self.module_scope.clone(),
         );
+
+        let function_doc = match &function.documentation {
+            None => nil(),
+            Some((_, documentation)) => {
+                jsdoc_comment(documentation, function.publicity).append(line())
+            }
+        };
+
         let head = if function.publicity.is_private() {
             "function "
         } else {
             "export function "
         };
 
-        let body = match generator.function_body(&function.body, function.arguments.as_slice()) {
-            // No error, let's continue!
-            Ok(body) => body,
+        let body = generator.function_body(&function.body, function.arguments.as_slice());
 
-            // There is an error coming from some expression that is not supported on JavaScript
-            // and the target support is not enforced. In this case we do not error, instead
-            // returning nothing which will cause no function to be generated.
-            Err(error) if error.is_unsupported() && !self.target_support.is_enforced() => {
-                return None;
-            }
-
-            // Some other error case which will be returned to the user.
-            Err(error) => return Some(Err(error)),
-        };
-
-        let document = docvec![
+        docvec![
+            function_doc,
             head,
             maybe_escape_identifier(name.as_str()),
-            fun_args(function.arguments.as_slice(), generator.tail_recursion_used),
+            fun_arguments(function.arguments.as_slice(), generator.tail_recursion_used),
             " {",
             docvec![line(), body].nest(INDENT).group(),
             line(),
             "}",
-        ];
-        Some(Ok(document))
+        ]
     }
 
     fn register_module_definitions_in_scope(&mut self) {
-        for statement in self.module.definitions.iter() {
-            match statement {
+        for definition in self.module.definitions.iter() {
+            match definition {
                 Definition::ModuleConstant(ModuleConstant { name, .. }) => {
                     self.register_in_scope(name)
                 }
@@ -672,80 +696,73 @@ impl<'a> Generator<'a> {
     }
 }
 
+fn jsdoc_comment(documentation: &EcoString, publicity: Publicity) -> Document<'_> {
+    let doc_lines = documentation
+        .trim_end()
+        .split('\n')
+        .map(|line| eco_format!(" *{line}", line = line.replace("*/", "*\\/")).to_doc())
+        .collect_vec();
+
+    // We start with the documentation of the function
+    let doc_body = join(doc_lines, line());
+    let mut doc = docvec!["/**", line(), doc_body, line()];
+    if !publicity.is_public() {
+        // If the function is not public we hide the documentation using
+        // the `@ignore` tag: https://jsdoc.app/tags-ignore
+        doc = docvec![doc, " * ", line(), " * @ignore", line()];
+    }
+    // And finally we close the doc comment
+    docvec![doc, " */"]
+}
+
 #[derive(Debug)]
 pub struct ModuleConfig<'a> {
     pub module: &'a TypedModule,
     pub line_numbers: &'a LineNumbers,
     pub src: &'a EcoString,
-    pub target_support: TargetSupport,
     pub typescript: TypeScriptDeclarations,
     pub stdlib_package: StdlibPackage,
     pub path: &'a Utf8Path,
     pub project_root: &'a Utf8Path,
 }
 
-pub fn module(config: ModuleConfig<'_>) -> Result<String, crate::Error> {
-    let path = config.path.to_path_buf();
-    let src = config.src.clone();
-    let document = Generator::new(config)
-        .compile()
-        .map_err(|error| crate::Error::JavaScript { path, src, error })?;
-    Ok(document.to_pretty_string(80))
+pub fn module(config: ModuleConfig<'_>) -> String {
+    let document = Generator::new(config).compile();
+    document.to_pretty_string(80)
 }
 
-pub fn ts_declaration(
-    module: &TypedModule,
-    path: &Utf8Path,
-    src: &EcoString,
-) -> Result<String, crate::Error> {
-    let document = typescript::TypeScriptGenerator::new(module)
-        .compile()
-        .map_err(|error| crate::Error::JavaScript {
-            path: path.to_path_buf(),
-            src: src.clone(),
-            error,
-        })?;
-    Ok(document.to_pretty_string(80))
+pub fn ts_declaration(module: &TypedModule) -> String {
+    let document = typescript::TypeScriptGenerator::new(module).compile();
+    document.to_pretty_string(80)
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub enum Error {
-    Unsupported { feature: String, location: SrcSpan },
-}
-
-impl Error {
-    /// Returns `true` if the error is [`Unsupported`].
-    ///
-    /// [`Unsupported`]: Error::Unsupported
-    #[must_use]
-    pub fn is_unsupported(&self) -> bool {
-        matches!(self, Self::Unsupported { .. })
-    }
-}
-
-fn fun_args(args: &'_ [TypedArg], tail_recursion_used: bool) -> Document<'_> {
+fn fun_arguments(arguments: &'_ [TypedArg], tail_recursion_used: bool) -> Document<'_> {
     let mut discards = 0;
-    wrap_args(args.iter().map(|a| match a.get_variable_name() {
-        None => {
-            let doc = if discards == 0 {
-                "_".to_doc()
-            } else {
-                eco_format!("_{discards}").to_doc()
-            };
-            discards += 1;
-            doc
-        }
-        Some(name) if tail_recursion_used => eco_format!("loop${name}").to_doc(),
-        Some(name) => maybe_escape_identifier(name).to_doc(),
-    }))
+    wrap_arguments(
+        arguments
+            .iter()
+            .map(|argument| match argument.get_variable_name() {
+                None => {
+                    let doc = if discards == 0 {
+                        "_".to_doc()
+                    } else {
+                        eco_format!("_{discards}").to_doc()
+                    };
+                    discards += 1;
+                    doc
+                }
+                Some(name) if tail_recursion_used => eco_format!("loop${name}").to_doc(),
+                Some(name) => maybe_escape_identifier(name).to_doc(),
+            }),
+    )
 }
 
-fn wrap_args<'a, I>(args: I) -> Document<'a>
+fn wrap_arguments<'a, I>(arguments: I) -> Document<'a>
 where
     I: IntoIterator<Item = Document<'a>>,
 {
     break_("", "")
-        .append(join(args, break_(",", ", ")))
+        .append(join(arguments, break_(",", ", ")))
         .nest(INDENT)
         .append(break_("", ""))
         .surround("(", ")")
@@ -851,7 +868,7 @@ fn is_usable_js_property(label: &str) -> bool {
         | "constructor"
         // `prototype` and `__proto__` to avoid unintentionally overriding the
         // prototype chain.
-        | "prototpye" | "__proto__" => false,
+        | "prototype" | "__proto__" => false,
         _ => true
     }
 }
@@ -936,7 +953,7 @@ pub(crate) fn bit_array_segment_int_value_to_bytes(
     mut value: BigInt,
     size: BigInt,
     endianness: Endianness,
-) -> Result<Vec<u8>, Error> {
+) -> Vec<u8> {
     // Clamp negative sizes to zero
     let size = size.max(BigInt::ZERO);
 
@@ -965,5 +982,5 @@ pub(crate) fn bit_array_segment_int_value_to_bytes(
         bytes.reverse();
     }
 
-    Ok(bytes)
+    bytes
 }
