@@ -1,5 +1,5 @@
 use hexpm::version::Version;
-use pubgrub::range::Range;
+use pubgrub::Range;
 
 use crate::{
     assert_js, assert_js_no_warnings_with_gleam_version, assert_js_warnings_with_gleam_version,
@@ -10,7 +10,7 @@ use crate::{
 fn empty() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<>>
 }
 "#,
@@ -21,7 +21,7 @@ fn go() {
 fn one() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<256>>
 }
 "#,
@@ -32,7 +32,7 @@ fn go() {
 fn two() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<256, 4>>
 }
 "#,
@@ -43,7 +43,7 @@ fn go() {
 fn integer() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<256:int>>
 }
 "#,
@@ -54,7 +54,7 @@ fn go() {
 fn float() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<1.1:float>>
 }
 "#,
@@ -65,7 +65,7 @@ fn go() {
 fn float_big_endian() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<1.1:float-big>>
 }
 "#,
@@ -76,7 +76,7 @@ fn go() {
 fn float_little_endian() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<1.1:float-little>>
 }
 "#,
@@ -87,7 +87,7 @@ fn go() {
 fn float_sized() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<1.1:float-32>>
 }
 "#,
@@ -98,7 +98,7 @@ fn go() {
 fn float_sized_big_endian() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<1.1:float-32-big>>
 }
 "#,
@@ -109,7 +109,7 @@ fn go() {
 fn float_sized_little_endian() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<1.1:float-32-little>>
 }
 "#,
@@ -120,7 +120,7 @@ fn go() {
 fn sized_constant_value() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<256:64>>
 }
 "#,
@@ -131,7 +131,7 @@ fn go() {
 fn sized_dynamic_value() {
     assert_js!(
         r#"
-fn go(i: Int) {
+pub fn go(i: Int) {
   <<i:64>>
 }
 "#,
@@ -142,7 +142,7 @@ fn go(i: Int) {
 fn sized_constant_value_positive_overflow() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<80_000:16>>
 }
 "#,
@@ -153,7 +153,7 @@ fn go() {
 fn sized_constant_value_negative_overflow() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<-80_000:16>>
 }
 "#,
@@ -164,7 +164,7 @@ fn go() {
 fn sized_constant_value_max_size_for_compile_time_evaluation() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<-1:48>>
 }
 "#,
@@ -175,7 +175,7 @@ fn go() {
 fn sized_big_endian_constant_value() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<256:16-big>>
 }
 "#,
@@ -186,7 +186,7 @@ fn go() {
 fn sized_big_endian_dynamic_value() {
     assert_js!(
         r#"
-fn go(i: Int) {
+pub fn go(i: Int) {
   <<i:16-big>>
 }
 "#,
@@ -197,7 +197,7 @@ fn go(i: Int) {
 fn sized_little_endian_constant_value() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<256:16-little>>
 }
 "#,
@@ -208,7 +208,7 @@ fn go() {
 fn sized_little_endian_dynamic_value() {
     assert_js!(
         r#"
-fn go(i: Int) {
+pub fn go(i: Int) {
   <<i:16-little>>
 }
 "#,
@@ -219,7 +219,7 @@ fn go(i: Int) {
 fn explicit_sized_constant_value() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   <<256:size(32)>>
 }
 "#,
@@ -230,7 +230,7 @@ fn go() {
 fn explicit_sized_dynamic_value() {
     assert_js!(
         r#"
-fn go(i: Int) {
+pub fn go(i: Int) {
   <<i:size(32)>>
 }
 "#,
@@ -241,7 +241,7 @@ fn go(i: Int) {
 fn variable_sized() {
     assert_js!(
         r#"
-fn go(x, y) {
+pub fn go(x, y) {
   <<x:size(y)>>
 }
 "#,
@@ -252,7 +252,7 @@ fn go(x, y) {
 fn variable() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   <<256, 4, x>>
 }
 "#,
@@ -263,7 +263,7 @@ fn go(x) {
 fn utf8() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   <<256, 4, x, "Gleam":utf8>>
 }
 "#,
@@ -274,7 +274,7 @@ fn go(x) {
 fn match_utf8_with_escape_chars() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<"\"\\\r\n\t\f\u{1f600}">> = x
 }
 "#,
@@ -285,7 +285,7 @@ fn go(x) {
 fn match_utf8() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<"Gleam 👍":utf8>> = x
 }
 "#,
@@ -296,7 +296,7 @@ fn go(x) {
 fn match_case_utf8_with_escape_chars() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<"\"\\\r\n\t\f\u{1f600}">> -> 1
     _ -> 2
@@ -310,7 +310,7 @@ fn go(x) {
 fn match_case_utf8() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<"Gleam 👍":utf8>> -> 1
     _ -> 2
@@ -324,7 +324,7 @@ fn go(x) {
 fn utf8_codepoint() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   <<x:utf8_codepoint, "Gleam":utf8>>
 }
 "#,
@@ -346,7 +346,7 @@ pub fn go(x) {
 fn bit_string() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   <<x:bits>>
 }
 "#,
@@ -357,7 +357,7 @@ fn go(x) {
 fn bits() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   <<x:bits>>
 }
 "#,
@@ -368,7 +368,7 @@ fn go(x) {
 fn bit_array_sliced() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   <<<<0xAB>>:bits-4>>
 }
 "#,
@@ -379,7 +379,7 @@ fn go(x) {
 fn bit_array_dynamic_slice() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let i = 4
   <<<<0xAB>>:bits-size(i)>>
 }
@@ -413,7 +413,7 @@ pub fn go(x) {
 fn empty_match() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<>> = x
 }
 "#,
@@ -424,7 +424,7 @@ fn go(x) {
 fn case_empty_match() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<>> -> 1
     _ -> 2
@@ -438,7 +438,7 @@ fn go(x) {
 fn match_bytes() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1, y>> = x
 }
 "#,
@@ -449,7 +449,7 @@ fn go(x) {
 fn case_match_bytes() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1, y>> -> y
     _ -> 1
@@ -463,7 +463,7 @@ fn go(x) {
 fn match_sized() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:16, b:8>> = x
 }
 "#,
@@ -474,7 +474,7 @@ fn go(x) {
 fn case_match_sized() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:16, b:8>> -> a + b
     _ -> 1
@@ -488,7 +488,7 @@ fn go(x) {
 fn match_sized_unaligned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:17, b:7>> = x
 }
 "#,
@@ -499,7 +499,7 @@ fn go(x) {
 fn case_match_sized_unaligned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:17, b:7>> -> b * 2
     _ -> 1
@@ -513,7 +513,7 @@ fn go(x) {
 fn match_sized_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1234:16, 123:8>> = x
 }
 "#,
@@ -524,7 +524,7 @@ fn go(x) {
 fn case_match_sized_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1234:16, 123:8>> -> 1
     _ -> 2
@@ -538,7 +538,7 @@ fn go(x) {
 fn match_unsigned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:unsigned>> = x
 }
 "#,
@@ -549,7 +549,7 @@ fn go(x) {
 fn case_match_unsigned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:unsigned>> -> a
     _ -> 1
@@ -563,7 +563,7 @@ fn go(x) {
 fn match_unsigned_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<-2:unsigned>> = x
 }
 "#,
@@ -574,7 +574,7 @@ fn go(x) {
 fn case_match_unsigned_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<-2:unsigned>> -> 1
     _ -> 2
@@ -588,7 +588,7 @@ fn go(x) {
 fn match_signed() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:signed>> = x
 }
 "#,
@@ -599,7 +599,7 @@ fn go(x) {
 fn case_match_signed() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:signed>> -> a
     _ -> 1
@@ -613,7 +613,7 @@ fn go(x) {
 fn match_signed_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<-1:signed>> = x
 }
 "#,
@@ -624,7 +624,7 @@ fn go(x) {
 fn case_match_signed_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<-1:signed>> -> 1
     _ -> 2
@@ -638,7 +638,7 @@ fn go(x) {
 fn match_sized_big_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:16-big>> = x
 }
 "#,
@@ -649,7 +649,7 @@ fn go(x) {
 fn case_match_sized_big_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:16-big>> -> a
     _ -> 1
@@ -663,7 +663,7 @@ fn go(x) {
 fn match_sized_big_endian_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1234:16-big>> = x
 }
 "#,
@@ -674,7 +674,7 @@ fn go(x) {
 fn case_match_sized_big_endian_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1234:16-big>> -> 1
     _ -> 2
@@ -688,7 +688,7 @@ fn go(x) {
 fn match_sized_little_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:16-little>> = x
 }
 "#,
@@ -699,7 +699,7 @@ fn go(x) {
 fn case_match_sized_little_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:16-little>> -> a
     _ -> 1
@@ -713,7 +713,7 @@ fn go(x) {
 fn match_sized_little_endian_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1234:16-little>> = x
 }
 "#,
@@ -724,7 +724,7 @@ fn go(x) {
 fn case_match_sized_little_endian_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1234:16-little>> -> 1
     _ -> 2
@@ -738,7 +738,7 @@ fn go(x) {
 fn match_sized_big_endian_unsigned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:16-big-unsigned>> = x
 }
 "#,
@@ -749,7 +749,7 @@ fn go(x) {
 fn case_match_sized_big_endian_unsigned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:16-big-unsigned>> -> a
     _ -> 1
@@ -763,7 +763,7 @@ fn go(x) {
 fn match_sized_big_endian_unsigned_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1234:16-big-unsigned>> = x
 }
 "#,
@@ -774,7 +774,7 @@ fn go(x) {
 fn case_match_sized_big_endian_unsigned_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1234:16-big-unsigned>> -> 1
     _ -> 2
@@ -788,7 +788,7 @@ fn go(x) {
 fn match_sized_big_endian_signed() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:16-big-signed>> = x
 }
 "#,
@@ -799,7 +799,7 @@ fn go(x) {
 fn case_match_sized_big_endian_signed() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:16-big-signed>> -> a
     _ -> 1
@@ -813,7 +813,7 @@ fn go(x) {
 fn match_sized_big_endian_signed_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1234:16-big-signed>> = x
 }
 "#,
@@ -824,7 +824,7 @@ fn go(x) {
 fn case_match_sized_big_endian_signed_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1234:16-big-signed>> -> 1
     _ -> 2
@@ -838,7 +838,7 @@ fn go(x) {
 fn match_sized_little_endian_unsigned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:16-little-unsigned>> = x
 }
 "#,
@@ -849,7 +849,7 @@ fn go(x) {
 fn case_match_sized_little_endian_unsigned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:16-little-unsigned>> -> a
     _ -> 1
@@ -863,7 +863,7 @@ fn go(x) {
 fn match_sized_little_endian_unsigned_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1234:16-little-unsigned>> = x
 }
 "#,
@@ -874,7 +874,7 @@ fn go(x) {
 fn case_match_sized_little_endian_unsigned_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1234:16-little-unsigned>> -> 1
     _ -> 2
@@ -888,7 +888,7 @@ fn go(x) {
 fn match_sized_little_endian_signed() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:16-little-signed>> = x
 }
 "#,
@@ -899,7 +899,7 @@ fn go(x) {
 fn case_match_sized_little_endian_signed() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:16-little-signed>> -> a
     _ -> 1
@@ -913,7 +913,7 @@ fn go(x) {
 fn match_sized_little_endian_signed_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1234:16-little-signed>> = x
 }
 "#,
@@ -924,7 +924,7 @@ fn go(x) {
 fn case_match_sized_little_endian_signed_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1234:16-little-signed>> -> 1
     _ -> 2
@@ -938,7 +938,7 @@ fn go(x) {
 fn match_dynamic_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   let assert <<a:size(n)>> = x
 }
@@ -950,7 +950,7 @@ fn go(x) {
 fn case_match_dynamic_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   case x {
     <<a:size(n)>> -> a
@@ -965,7 +965,7 @@ fn go(x) {
 fn match_dynamic_size_with_other_segments() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   let m = 32
   let assert <<first:size(8), a:size(n), b:size(m), rest:bits>> = x
@@ -978,7 +978,7 @@ fn go(x) {
 fn case_match_dynamic_size_with_other_segments() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   let m = 32
   case x {
@@ -994,7 +994,7 @@ fn go(x) {
 fn match_dynamic_size_shadowed_variable() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   let n = 5
   let assert <<a:size(n)>> = x
@@ -1007,7 +1007,7 @@ fn go(x) {
 fn case_match_dynamic_size_shadowed_variable() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   let n = 5
   case x {
@@ -1023,7 +1023,7 @@ fn go(x) {
 fn match_dynamic_size_literal_value() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 8
   let assert <<a:size(n), 0b010101:size(8)>> = x
 }
@@ -1035,7 +1035,7 @@ fn go(x) {
 fn case_match_dynamic_size_literal_value() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 8
   case x {
     <<a:size(n), 0b010101:size(8)>> -> a
@@ -1050,7 +1050,7 @@ fn go(x) {
 fn match_dynamic_bits_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   let assert <<a:bits-size(n)>> = x
 }
@@ -1062,7 +1062,7 @@ fn go(x) {
 fn case_match_dynamic_bits_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 16
   case x {
     <<a:bits-size(n)>> -> a
@@ -1077,7 +1077,7 @@ fn go(x) {
 fn match_dynamic_bytes_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 3
   let assert <<a:bytes-size(n)>> = x
 }
@@ -1089,7 +1089,7 @@ fn go(x) {
 fn case_match_dynamic_bytes_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 3
   case x {
     <<a:bytes-size(n)>> -> a
@@ -1104,7 +1104,7 @@ fn go(x) {
 fn discard_sized() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_:16, _:8>> = x
   let assert <<_:16-little-signed, _:8>> = x
 }
@@ -1116,7 +1116,7 @@ fn go(x) {
 fn case_discard_sized() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_:16, _:8>> -> 1
     _ -> 2
@@ -1134,7 +1134,7 @@ fn go(x) {
 fn match_sized_value() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<i:16>> = x
 }
 "#,
@@ -1145,7 +1145,7 @@ fn go(x) {
 fn case_match_sized_value() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<i:16>> -> i
     _ -> 1
@@ -1159,7 +1159,7 @@ fn go(x) {
 fn match_sized_value_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<258:16>> = x
 }
 "#,
@@ -1170,7 +1170,7 @@ fn go(x) {
 fn case_match_sized_value_constant_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<258:16>> -> 1
     _ -> 2
@@ -1184,7 +1184,7 @@ fn go(x) {
 fn match_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:float, b:int>> = x
 }
 "#,
@@ -1195,7 +1195,7 @@ fn go(x) {
 fn case_match_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:float, b:int>> -> #(a, b)
     _ -> #(1.1, 2)
@@ -1209,7 +1209,7 @@ fn go(x) {
 fn match_float_big_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:float-big, b:int>> = x
 }
 "#,
@@ -1220,7 +1220,7 @@ fn go(x) {
 fn case_match_float_big_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:float-big, b:int>> -> #(a, b)
     _ -> #(1.1, 1)
@@ -1234,7 +1234,7 @@ fn go(x) {
 fn match_float_little_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:float-little, b:int>> = x
 }
 "#,
@@ -1245,7 +1245,7 @@ fn go(x) {
 fn case_match_float_little_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:float-little, b:int>> -> #(a, b)
     _ -> #(1.1, 2)
@@ -1259,7 +1259,7 @@ fn go(x) {
 fn match_float_sized() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:float-32, b:int>> = x
 }
 "#,
@@ -1270,7 +1270,7 @@ fn go(x) {
 fn case_match_float_sized() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:float-32, b:int>> -> #(a, b)
     _ -> #(1.1, 2)
@@ -1284,7 +1284,7 @@ fn go(x) {
 fn match_float_sized_big_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:float-32-big, b:int>> = x
 }
 "#,
@@ -1295,7 +1295,7 @@ fn go(x) {
 fn case_match_float_sized_big_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:float-32-big, b:int>> -> #(a, b)
     _ -> #(1.1, 2)
@@ -1309,7 +1309,7 @@ fn go(x) {
 fn match_float_sized_little_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:float-32-little, b:int>> = x
 }
 "#,
@@ -1320,7 +1320,7 @@ fn go(x) {
 fn case_match_float_sized_little_endian() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:float-32-little, b:int>> -> #(a, b)
     _ -> #(1.1, 2)
@@ -1334,7 +1334,7 @@ fn go(x) {
 fn match_literal_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1.4, b:int>> = x
 }
 "#,
@@ -1345,7 +1345,7 @@ fn go(x) {
 fn case_match_literal_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1.4, b:int>> -> 1
     _ -> 2
@@ -1359,7 +1359,7 @@ fn go(x) {
 fn match_literal_unaligned_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 1
   let assert <<_:size(n), 1.1, _:bits>> = x
 }
@@ -1371,7 +1371,7 @@ fn go(x) {
 fn case_match_literal_unaligned_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 1
   case x {
     <<_:size(n), 1.1, _:int>> -> 1
@@ -1386,7 +1386,7 @@ fn go(x) {
 fn match_literal_aligned_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_, 1.1, _:bits>> = x
 }
 "#,
@@ -1397,7 +1397,7 @@ fn go(x) {
 fn case_match_literal_aligned_float() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_, 1.1, _:int>> -> 1
     _ -> 2
@@ -1411,7 +1411,7 @@ fn go(x) {
 fn match_float_16_bit() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<a:float-size(16)>> = x
 }
 "#
@@ -1422,7 +1422,7 @@ fn go(x) {
 fn case_match_float_16_bit() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<a:float-size(16)>> -> a
     _ -> 1.1
@@ -1436,7 +1436,7 @@ fn go(x) {
 fn match_rest() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_, b:bytes>> = <<1,2,3>>
 }
 "#,
@@ -1447,7 +1447,7 @@ fn go(x) {
 fn case_match_rest() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case <<1, 2, 3>> {
     <<_, b:bytes>> -> b
     _ -> x
@@ -1461,7 +1461,7 @@ fn go(x) {
 fn match_bytes_with_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<f:bytes-2>> = <<1, 2>>
 }
 "#,
@@ -1472,7 +1472,7 @@ fn go(x) {
 fn case_match_bytes_with_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case <<1, 2>> {
     <<f:bytes-2>> -> f
     _ -> x
@@ -1486,7 +1486,7 @@ fn go(x) {
 fn match_bits_with_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_:4, f:bits-2, _:1>> = <<0x77:7>>
 }
 "#,
@@ -1497,7 +1497,7 @@ fn go(x) {
 fn case_match_bits_with_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case <<0x77:7>> {
     <<_:4, f:bits-2, _:1>> -> f
     _ -> x
@@ -1511,7 +1511,7 @@ fn go(x) {
 fn match_rest_bytes() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_, b:bytes>> = <<1,2,3>>
 }
 "#,
@@ -1522,7 +1522,7 @@ fn go(x) {
 fn case_match_rest_bytes() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_, b:bytes>> -> b
     _ -> x
@@ -1536,7 +1536,7 @@ fn go(x) {
 fn match_rest_bits() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_, b:bits>> = <<1,2,3>>
 }
 "#,
@@ -1547,7 +1547,7 @@ fn go(x) {
 fn case_match_rest_bits() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_, b:bits>> -> b
     _ -> x
@@ -1561,7 +1561,7 @@ fn go(x) {
 fn match_rest_bits_unaligned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_:5, b:bits>> = <<1,2,3>>
 }
 "#,
@@ -1572,7 +1572,7 @@ fn go(x) {
 fn case_match_rest_bits_unaligned() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_:5, b:bits>> -> b
     _ -> x
@@ -1586,7 +1586,7 @@ fn go(x) {
 fn match_binary_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<_, a:2-bytes>> = x
   let assert <<_, b:bytes-size(2)>> = x
 }
@@ -1598,7 +1598,7 @@ fn go(x) {
 fn case_match_binary_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_, a:2-bytes>> -> a
     _ -> x
@@ -1739,7 +1739,7 @@ fn as_module_const() {
 
 #[test]
 fn bit_array_literal_string_constant_is_treated_as_utf8() {
-    assert_js!(r#"const a = <<"hello", " ", "world">>"#);
+    assert_js!(r#"pub const a = <<"hello", " ", "world">>"#);
 }
 
 #[test]
@@ -1769,7 +1769,7 @@ pub fn main() {
 fn with_unit() {
     assert_js!(
         r#"
-fn main() {
+pub fn main() {
   <<1:size(2)-unit(2), 2:size(3)-unit(4)>>
 }
 "#,
@@ -1780,7 +1780,7 @@ fn main() {
 fn dynamic_size_with_unit() {
     assert_js!(
         r#"
-fn main() {
+pub fn main() {
   let size = 3
   <<1:size(size)-unit(2)>>
 }
@@ -1792,7 +1792,7 @@ fn main() {
 fn pattern_with_unit() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert <<1:size(2)-unit(2), 2:size(3)-unit(4)>> = x
 }
 "#,
@@ -1803,7 +1803,7 @@ fn go(x) {
 fn case_pattern_with_unit() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<1:size(2)-unit(2), 2:size(3)-unit(4)>> -> 1
     _ -> 2
@@ -1817,7 +1817,7 @@ fn go(x) {
 fn dynamic_size_pattern_with_unit() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let size = 3
   let assert <<1:size(size)-unit(2)>> = x
 }
@@ -1829,7 +1829,7 @@ fn go(x) {
 fn case_dynamic_size_pattern_with_unit() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let size = 3
   case x {
     <<1:size(size)-unit(2)>> -> 1
@@ -1844,7 +1844,7 @@ fn go(x) {
 fn case_dynamic_size_float_pattern_with_unit() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let size = 3
   case x {
     <<1.3:size(size)-unit(2)>> -> 1
@@ -1859,7 +1859,7 @@ fn go(x) {
 fn case_with_remaining_bytes_after_constant_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_, _, _:bytes>> -> 1
     _ -> 2
@@ -1873,7 +1873,7 @@ fn go(x) {
 fn case_with_remaining_bytes_after_variable_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 1
   case x {
     <<_:size(n), _, _:bytes>> -> 1
@@ -1888,7 +1888,7 @@ fn go(x) {
 fn case_with_remaining_bytes_after_variable_size_2() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 1
   case x {
     <<m:size(n), _:size(m), _:bytes>> -> 1
@@ -1903,7 +1903,7 @@ fn go(x) {
 fn case_is_byte_aligned() {
     assert_js!(
         r#"
-fn is_byte_aligned(x) {
+pub fn is_byte_aligned(x) {
   case x {
     <<_:bytes>> -> True
     _ -> False
@@ -1917,7 +1917,7 @@ fn is_byte_aligned(x) {
 fn alternative_patterns_with_variable_size() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<_, n, rest:size(n)>> |
     <<n, _, rest:size(n)>> -> True
@@ -1932,7 +1932,7 @@ fn go(x) {
 fn variable_sized_segment() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<n, rest:size(n)>> -> 1
     _ -> 2
@@ -1946,7 +1946,7 @@ fn go(x) {
 fn segments_shadowing_each_other() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = 1
   case x {
     <<n, rest:size(n)>> -> 1
@@ -1961,7 +1961,7 @@ fn go(x) {
 fn negative_size_pattern() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let n = -10
   case x {
     <<int:size(n)>> -> int
@@ -1976,7 +1976,7 @@ fn go(x) {
 fn negative_size_pattern_2() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   case x {
     <<n:signed, int:size(n)>> -> int
     _ -> 2
@@ -2494,3 +2494,140 @@ pub fn go(x) {
 "#
     );
 }
+
+#[test]
+fn unit_with_bits_option() {
+    assert_js!(
+        "
+pub fn go(x) {
+  <<x:bits-size(4)-unit(8)>>
+}
+"
+    );
+}
+
+#[test]
+fn unit_with_bits_option_constant() {
+    assert_js!(
+        "
+pub const bits = <<1, 2, 3>>
+pub const more_bits = <<bits:bits-size(3)-unit(8)>>
+"
+    );
+}
+
+#[test]
+fn operator_in_size_for_bit_array_segment() {
+    assert_js!(
+        "
+pub fn go(x) {
+  <<x:bits-size(4 + 1)>>
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4712
+#[test]
+fn multiple_variable_size_segments() {
+    assert_js!(
+        "
+pub fn main() {
+  let assert <<a, b:size(a), c:size(b)>> = <<1, 2, 3, 4>>
+  a + b + c
+}
+"
+    );
+}
+
+#[test]
+fn utf16_codepoint_little_endian() {
+    assert_js!(
+        "
+pub fn go(codepoint) {
+  <<codepoint:utf16_codepoint-little>>
+}
+"
+    );
+}
+
+#[test]
+fn utf32_codepoint_little_endian() {
+    assert_js!(
+        "
+pub fn go(codepoint) {
+  <<codepoint:utf32_codepoint-little>>
+}
+"
+    );
+}
+
+#[test]
+fn operator_in_pattern_size() {
+    assert_js!(
+        "
+pub fn main() {
+  let assert <<len, payload:size(len * 8)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn operator_in_pattern_size2() {
+    assert_js!(
+        "
+pub fn main() {
+  let assert <<len, payload:size(len / 8 - 1)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn operator_in_pattern_size3() {
+    assert_js!(
+        "
+pub fn main() {
+  let additional = 10
+  let assert <<len, payload:size(len + additional * 8)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn block_in_pattern_size() {
+    assert_js!(
+        "
+pub fn main() {
+  let assert <<len, payload:size({ len - 1 } * 8)>> = <<>>
+}
+"
+    );
+}
+
+#[test]
+fn non_byte_aligned_size_calculation() {
+    assert_js!(
+        "
+pub fn main() {
+  case <<>> {
+    <<a:1, b:3, c:size(b - 2)>> -> c + b
+    _ -> 1
+  }
+}
+"
+    );
+}
+
+#[test]
+fn pattern_match_on_negative_size_calculation() {
+    assert_js!(
+        "
+pub fn main() {
+  let assert <<a, b:size(a - 100000), c:size(b)>> = <<1, 2, 3, 4, 5>>
+}
+"
+    );
+}
