@@ -4,15 +4,15 @@ use crate::assert_js;
 fn int_literals() {
     assert_js!(
         r#"
-fn go() {
-    1
-    2
-    -3
-    4001
-    0b00001111
-    0o17
-    0xF
-    1_000
+pub fn go() {
+  1
+  2
+  -3
+  4001
+  0b00001111
+  0o17
+  0xF
+  1_000
 }
 "#,
     );
@@ -22,11 +22,11 @@ fn go() {
 fn float_literals() {
     assert_js!(
         r#"
-fn go() {
-    1.5
-    2.0
-    -0.1
-    1.
+pub fn go() {
+  1.5
+  2.0
+  -0.1
+  1.
 }
 "#,
     );
@@ -36,15 +36,15 @@ fn go() {
 fn float_scientific_literals() {
     assert_js!(
         r#"
-fn go() {
-    0.01e-1
-    0.01e-0
-    -10.01e-1
-    -10.01e-0
-    100.001e523
-    -100.001e-523
-    100.001e123_456_789
-    -100.001e-123_456_789
+pub fn go() {
+  0.01e-1
+  0.01e-0
+  -10.01e-1
+  -10.01e-0
+  100.001e523
+  -100.001e-523
+  100.001e123_456_789
+  -100.001e-123_456_789
 }
 "#,
     );
@@ -54,16 +54,16 @@ fn go() {
 fn int_operators() {
     assert_js!(
         r#"
-fn go() {
-    1 + 1 // => 2
-    5 - 1 // => 4
-    5 / 2 // => 2
-    3 * 3 // => 9
-    5 % 2 // => 1
-    2 > 1  // => True
-    2 < 1  // => False
-    2 >= 1 // => True
-    2 <= 1 // => False
+pub fn go() {
+  1 + 1 // => 2
+  5 - 1 // => 4
+  5 / 2 // => 2
+  3 * 3 // => 9
+  5 % 2 // => 1
+  2 > 1  // => True
+  2 < 1  // => False
+  2 >= 1 // => True
+  2 <= 1 // => False
 }
 "#,
     );
@@ -73,7 +73,7 @@ fn go() {
 fn int_divide_complex_expr() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   case 1 >= 0 {
     True -> 2
     False -> 4
@@ -87,7 +87,7 @@ fn go() {
 fn int_mod_complex_expr() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   case 1 >= 0 {
     True -> 2
     False -> 4
@@ -101,7 +101,7 @@ fn go() {
 fn float_operators() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
     1.0 +. 1.4 // => 2.4
     5.0 -. 1.5 // => 3.5
     5.0 /. 2.0 // => 2.5
@@ -120,7 +120,7 @@ fn go() {
 fn float_divide_complex_expr() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   case 1.0 >=. 0.0 {
     True -> 2.0
     False -> 4.0
@@ -134,7 +134,7 @@ fn go() {
 fn wide_float_div() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   111111111111111111111111111111. /. 22222222222222222222222222222222222.
 }
 "#,
@@ -145,7 +145,7 @@ fn go() {
 fn int_patterns() {
     assert_js!(
         r#"
-fn go(x) {
+pub fn go(x) {
   let assert 4 = x
 }
 "#,
@@ -156,7 +156,7 @@ fn go(x) {
 fn int_equality() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   1 != 2
   1 == 2
 }
@@ -168,7 +168,7 @@ fn go() {
 fn int_equality1() {
     assert_js!(
         r#"
-fn go(y) {
+pub fn go(y) {
   let x = 1
   x == y
 }
@@ -180,7 +180,7 @@ fn go(y) {
 fn float_equality() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   1.0 != 2.0
   1.0 == 2.0
 }
@@ -192,7 +192,7 @@ fn go() {
 fn float_equality1() {
     assert_js!(
         r#"
-fn go(y) {
+pub fn go(y) {
   let x = 1.0
   x == y
 }
@@ -204,7 +204,7 @@ fn go(y) {
 fn operator_precedence() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   2.4 *. { 3.5 +. 6.0 }
 }
 "#,
@@ -215,7 +215,7 @@ fn go() {
 fn remainder() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   5 % 0 // => 0
 }
 "#,
@@ -226,7 +226,7 @@ fn go() {
 fn int_negation() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   let a = 3
   let b = -a
 }
@@ -238,7 +238,7 @@ fn go() {
 fn repeated_int_negation() {
     assert_js!(
         r#"
-fn go() {
+pub fn go() {
   let a = 3
   let b = --a
 }
@@ -251,7 +251,7 @@ fn go() {
 fn preceeding_zeros_int() {
     assert_js!(
         r#"
-fn main() {
+pub fn main() {
   09_179
 }
 "#
@@ -263,7 +263,7 @@ fn main() {
 fn preceeding_zeros_float() {
     assert_js!(
         r#"
-fn main() {
+pub fn main() {
   09_179.1
 }
 "#
@@ -275,7 +275,7 @@ fn main() {
 fn preceeding_zeros_int_const() {
     assert_js!(
         r#"
-const x = 09_179
+pub const x = 09_179
 "#
     );
 }
@@ -285,7 +285,7 @@ const x = 09_179
 fn preceeding_zeros_float_const() {
     assert_js!(
         r#"
-const x = 09_179.1
+pub const x = 09_179.1
 "#
     );
 }
@@ -295,7 +295,7 @@ const x = 09_179.1
 fn preceeding_zeros_int_pattern() {
     assert_js!(
         r#"
-fn main(x) {
+pub fn main(x) {
   let assert 09_179 = x
 }
 "#
@@ -307,7 +307,7 @@ fn main(x) {
 fn preceeding_zeros_float_pattern() {
     assert_js!(
         r#"
-fn main(x) {
+pub fn main(x) {
   let assert 09_179.1 = x
 }
 "#
@@ -430,3 +430,84 @@ pub fn main() {
 "
     );
 }
+
+#[test]
+fn division_by_zero_float() {
+    assert_js!(
+        "pub fn main() {
+  1.1 /. 0.0
+}"
+    )
+}
+
+#[test]
+fn division_by_non_zero_float() {
+    assert_js!(
+        "pub fn main() {
+  1.1 /. 2.3
+}"
+    )
+}
+
+#[test]
+fn complex_division_by_non_zero_float() {
+    assert_js!(
+        "pub fn main() {
+  { 1.1 +. 2.0 } /. 2.3
+}"
+    )
+}
+
+#[test]
+fn division_by_zero_int() {
+    assert_js!(
+        "pub fn main() {
+  1 / 0
+}"
+    )
+}
+
+#[test]
+fn division_by_non_zero_int() {
+    assert_js!(
+        "pub fn main() {
+  1 / 2
+}"
+    )
+}
+
+#[test]
+fn complex_division_by_non_zero_int() {
+    assert_js!(
+        "pub fn main() {
+  { 1 + 2 } / 3
+}"
+    )
+}
+
+#[test]
+fn remainder_by_zero_int() {
+    assert_js!(
+        "pub fn main() {
+  1 % 0
+}"
+    )
+}
+
+#[test]
+fn remainder_by_non_zero_int() {
+    assert_js!(
+        "pub fn main() {
+  1 % 2
+}"
+    )
+}
+
+#[test]
+fn complex_remainder_by_non_zero_int() {
+    assert_js!(
+        "pub fn main() {
+  { 1 + 2 } % 3
+}"
+    )
+}
