@@ -12,14 +12,15 @@ use crate::{
     config::PackageConfig,
     io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     language_server::{
-        compiler::LspProjectCompiler, files::FileSystemProxy, progress::ProgressReporter,
+        code_action::RemoveBlock, compiler::LspProjectCompiler, files::FileSystemProxy,
+        progress::ProgressReporter,
     },
     line_numbers::LineNumbers,
     paths::ProjectPaths,
     type_::{
         self, Deprecation, ModuleInterface, Type, TypeConstructor, ValueConstructor,
         ValueConstructorVariant,
-        error::{Named, VariableOrigin},
+        error::{Named, VariableSyntax},
         printer::Printer,
     },
 };
@@ -293,12 +294,12 @@ where
                 }
                 | Located::Constant(Constant::String { .. }) => None,
                 Located::Expression {
-                    expression: TypedExpr::Call { fun, args, .. },
+                    expression: TypedExpr::Call { fun, arguments, .. },
                     ..
                 } => {
                     let mut completions = vec![];
                     completions.append(&mut completer.completion_values());
-                    completions.append(&mut completer.completion_labels(fun, args));
+                    completions.append(&mut completer.completion_labels(fun, arguments));
                     Some(completions)
                 }
                 Located::Expression {
@@ -435,6 +436,7 @@ where
             );
             actions.extend(InlineVariable::new(module, &lines, &params).code_actions());
             actions.extend(WrapInBlock::new(module, &lines, &params).code_actions());
+            actions.extend(RemoveBlock::new(module, &lines, &params).code_actions());
             GenerateDynamicDecoder::new(module, &lines, &params, &mut actions).code_actions();
             GenerateJsonEncoder::new(
                 module,
@@ -637,12 +639,12 @@ where
             Ok(match reference_for_ast_node(found, &current_module.name) {
                 Some(Referenced::LocalVariable {
                     location, origin, ..
-                }) if location.contains(byte_index) => match origin {
-                    Some(VariableOrigin::Generated) => None,
+                }) if location.contains(byte_index) => match origin.map(|origin| origin.syntax) {
+                    Some(VariableSyntax::Generated) => None,
                     Some(
-                        VariableOrigin::Variable(_)
-                        | VariableOrigin::AssignmentPattern
-                        | VariableOrigin::LabelShorthand(_),
+                        VariableSyntax::Variable { .. }
+                        | VariableSyntax::AssignmentPattern
+                        | VariableSyntax::LabelShorthand(_),
                     )
                     | None => success_response(location),
                 },
@@ -693,17 +695,27 @@ where
                 Some(Referenced::LocalVariable {
                     origin,
                     definition_location,
+                    name,
                     ..
                 }) => {
-                    let rename_kind = match origin {
-                        Some(VariableOrigin::Generated) => return Ok(None),
-                        Some(VariableOrigin::LabelShorthand(_)) => {
+                    let rename_kind = match origin.map(|origin| origin.syntax) {
+                        Some(VariableSyntax::Generated) => return Ok(None),
+                        Some(VariableSyntax::LabelShorthand(_)) => {
                             VariableReferenceKind::LabelShorthand
                         }
-                        Some(VariableOrigin::AssignmentPattern | VariableOrigin::Variable(_))
+                        Some(
+                            VariableSyntax::AssignmentPattern | VariableSyntax::Variable { .. },
+                        )
                         | None => VariableReferenceKind::Variable,
                     };
-                    rename_local_variable(module, &lines, &params, definition_location, rename_kind)
+                    rename_local_variable(
+                        module,
+                        &lines,
+                        &params,
+                        definition_location,
+                        name,
+                        rename_kind,
+                    )
                 }
                 Some(Referenced::ModuleValue {
                     module: module_name,
@@ -772,16 +784,17 @@ where
                     origin,
                     definition_location,
                     location,
-                }) if location.contains(byte_index) => match origin {
-                    Some(VariableOrigin::Generated) => None,
+                    name,
+                }) if location.contains(byte_index) => match origin.map(|origin| origin.syntax) {
+                    Some(VariableSyntax::Generated) => None,
                     Some(
-                        VariableOrigin::LabelShorthand(_)
-                        | VariableOrigin::AssignmentPattern
-                        | VariableOrigin::Variable(_),
+                        VariableSyntax::LabelShorthand(_)
+                        | VariableSyntax::AssignmentPattern
+                        | VariableSyntax::Variable { .. },
                     )
                     | None => {
                         let variable_references =
-                            find_variable_references(&module.ast, definition_location);
+                            find_variable_references(&module.ast, definition_location, name);
 
                         let mut reference_locations =
                             Vec::with_capacity(variable_references.len() + 1);
@@ -1182,7 +1195,11 @@ fn hover_for_pattern(pattern: &TypedPattern, line_numbers: LineNumbers, module:
 
 fn get_function_type(fun: &TypedFunction) -> Type {
     Type::Fn {
-        args: fun.arguments.iter().map(|arg| arg.type_.clone()).collect(),
+        arguments: fun
+            .arguments
+            .iter()
+            .map(|argument| argument.type_.clone())
+            .collect(),
         return_: fun.return_type.clone(),
     }
 }
@@ -1563,12 +1580,17 @@ fn get_hexdocs_link_section(
     ast: &TypedModule,
     hex_deps: &HashSet<EcoString>,
 ) -> Option<String> {
-    let package_name = ast.definitions.iter().find_map(|def| match def {
-        Definition::Import(p) if p.module == module_name && hex_deps.contains(&p.package) => {
-            Some(&p.package)
-        }
-        _ => None,
-    })?;
+    let package_name = ast
+        .definitions
+        .iter()
+        .find_map(|definition| match definition {
+            Definition::Import(import)
+                if import.module == module_name && hex_deps.contains(&import.package) =>
+            {
+                Some(&import.package)
+            }
+            _ => None,
+        })?;
 
     Some(format_hexdocs_link_section(
         package_name,
