@@ -17,7 +17,7 @@ use crate::{
 pub fn for_expression(expr: &TypedExpr) -> Option<SignatureHelp> {
     // If we're inside a function call we can provide signature help,
     // otherwise we don't want anything to pop up.
-    let TypedExpr::Call { fun, args, .. } = expr else {
+    let TypedExpr::Call { fun, arguments, .. } = expr else {
         return None;
     };
 
@@ -27,7 +27,7 @@ pub fn for_expression(expr: &TypedExpr) -> Option<SignatureHelp> {
         // help.
         TypedExpr::Var {
             constructor, name, ..
-        } => signature_help(name.clone(), fun, args, constructor.field_map()),
+        } => signature_help(name.clone(), fun, arguments, constructor.field_map()),
 
         // If we're making a qualified call to another module's function
         // then we want to show its type, documentation and the exact name
@@ -50,7 +50,7 @@ pub fn for_expression(expr: &TypedExpr) -> Option<SignatureHelp> {
                 | ModuleValueConstructor::Fn { field_map, .. } => field_map.into(),
             };
             let name = format!("{module_alias}.{label}").into();
-            signature_help(name, fun, args, field_map)
+            signature_help(name, fun, arguments, field_map)
         }
 
         // If the function bein called is an invalid node we don't want to
@@ -67,7 +67,7 @@ pub fn for_expression(expr: &TypedExpr) -> Option<SignatureHelp> {
         //                  ^ When the cursor is here we are going to show
         //                    "fn(a: a) -> a" as the help signature.
         //
-        _ => signature_help("fn".into(), fun, args, None),
+        _ => signature_help("fn".into(), fun, arguments, None),
     }
 }
 
@@ -77,7 +77,7 @@ pub fn for_expression(expr: &TypedExpr) -> Option<SignatureHelp> {
 ///
 /// - `fun_name` is used as the display name of the function in the help
 ///   signature.
-/// - `supplied_args` are arguments being passed to the function call, those
+/// - `supplied_arguments` are arguments being passed to the function call, those
 ///   might not be of the correct arity or have wrong types but are used to
 ///   deduce which argument should be highlighted next in the help signature.
 /// - `field_map` is the function's field map (if any) that will be used to
@@ -87,13 +87,13 @@ pub fn for_expression(expr: &TypedExpr) -> Option<SignatureHelp> {
 fn signature_help(
     fun_name: EcoString,
     fun: &TypedExpr,
-    supplied_args: &[CallArg<TypedExpr>],
+    supplied_arguments: &[CallArg<TypedExpr>],
     field_map: Option<&FieldMap>,
 ) -> Option<SignatureHelp> {
-    let (args, return_) = fun.type_().fn_types()?;
+    let (arguments, return_) = fun.type_().fn_types()?;
 
     // If the function has no arguments, we don't want to show any help.
-    let arity = args.len() as u32;
+    let arity = arguments.len() as u32;
     if arity == 0 {
         return None;
     }
@@ -109,9 +109,9 @@ fn signature_help(
 
     let printer = Printer::new();
     let (label, parameters) =
-        print_signature_help(printer, fun_name, args, return_, &index_to_label);
+        print_signature_help(printer, fun_name, arguments, return_, &index_to_label);
 
-    let active_parameter = active_parameter_index(arity, supplied_args, index_to_label)
+    let active_parameter = active_parameter_index(arity, supplied_arguments, index_to_label)
         // If we don't want to highlight any arg in the suggestion we have to
         // explicitly provide an out of bound index.
         .or(Some(arity));
@@ -135,21 +135,21 @@ fn signature_help(
 
 fn active_parameter_index(
     arity: u32,
-    supplied_args: &[CallArg<TypedExpr>],
+    supplied_arguments: &[CallArg<TypedExpr>],
     mut index_to_label: HashMap<u32, &EcoString>,
 ) -> Option<u32> {
     let mut is_use_call = false;
-    let mut found_labelled_arg = false;
+    let mut found_labelled_argument = false;
     let mut used_labels = HashSet::new();
 
-    let mut supplied_unlabelled_args = 0;
-    let unlabelled_args = arity - index_to_label.len() as u32;
+    let mut supplied_unlabelled_arguments = 0;
+    let unlabelled_arguments = arity - index_to_label.len() as u32;
 
-    for (i, arg) in supplied_args.iter().enumerate() {
+    for (i, arg) in supplied_arguments.iter().enumerate() {
         // If there's an unlabelled argument after a labelled one, we can't
         // figure out what to suggest since arguments were passed in a wrong
         // order.
-        if found_labelled_arg && arg.label.is_none() && !arg.is_implicit() {
+        if found_labelled_argument && arg.label.is_none() && !arg.is_implicit() {
             return None;
         }
 
@@ -166,7 +166,7 @@ fn active_parameter_index(
 
         match &arg.label {
             Some(label) => {
-                found_labelled_arg = true;
+                found_labelled_argument = true;
                 let _ = used_labels.insert(label);
             }
 
@@ -174,14 +174,14 @@ fn active_parameter_index(
             // corresponding to it from the field map since it has already been
             // passed as an unlabelled argument.
             None => {
-                supplied_unlabelled_args += 1;
+                supplied_unlabelled_arguments += 1;
                 let _ = index_to_label.remove(&(i as u32));
             }
         }
     }
 
-    let active_index = if supplied_unlabelled_args < unlabelled_args {
-        if found_labelled_arg {
+    let active_index = if supplied_unlabelled_arguments < unlabelled_arguments {
+        if found_labelled_argument {
             // If I have supplied some labelled args but I haven't supplied all
             // unlabelled args before a labelled one then we can't safely
             // suggest anything as the next argument.
@@ -189,7 +189,7 @@ fn active_parameter_index(
         } else {
             // If I haven't supplied enough unlabelled arguments then I have to
             // set the next one as active (be it labelled or not).
-            Some(supplied_unlabelled_args)
+            Some(supplied_unlabelled_arguments)
         }
     } else {
         // If I have supplied all the unlabelled arguments (and we could have
@@ -200,7 +200,7 @@ fn active_parameter_index(
             .filter(|(_index, label)| !used_labels.contains(label))
             .map(|(index, _label)| index)
             .min()
-            .or(Some(supplied_args.len() as u32))
+            .or(Some(supplied_arguments.len() as u32))
     };
 
     // If we're showing hints for a use call and we end up deciding that the
@@ -222,21 +222,21 @@ fn active_parameter_index(
 fn print_signature_help(
     mut printer: Printer,
     function_name: EcoString,
-    args: Vec<Arc<Type>>,
+    arguments: Vec<Arc<Type>>,
     return_: Arc<Type>,
     index_to_label: &HashMap<u32, &EcoString>,
 ) -> (String, Vec<ParameterInformation>) {
-    let args_count = args.len();
+    let arguments_count = arguments.len();
     let mut signature = format!("{function_name}(");
-    let mut parameter_informations = Vec::with_capacity(args_count);
+    let mut parameter_informations = Vec::with_capacity(arguments_count);
 
-    for (i, arg) in args.iter().enumerate() {
+    for (i, argument) in arguments.iter().enumerate() {
         let arg_start = signature.len();
         if let Some(label) = index_to_label.get(&(i as u32)) {
             signature.push_str(label);
             signature.push_str(": ");
         }
-        signature.push_str(&printer.pretty_print(arg, 0));
+        signature.push_str(&printer.pretty_print(argument, 0));
         let arg_end = signature.len();
         let label = ParameterLabel::LabelOffsets([arg_start as u32, arg_end as u32]);
 
@@ -245,7 +245,7 @@ fn print_signature_help(
             documentation: None,
         });
 
-        let is_last = i == args_count - 1;
+        let is_last = i == arguments_count - 1;
         if !is_last {
             signature.push_str(", ");
         }
