@@ -473,11 +473,31 @@ fn echo_followed_by_expression_ends_where_expression_ends() {
     assert_parse!("echo wibble");
 }
 
+#[test]
+fn echo_with_simple_expression_1() {
+    assert_parse!("echo wibble as message");
+}
+
+#[test]
+fn echo_with_simple_expression_2() {
+    assert_parse!("echo wibble as \"message\"");
+}
+
+#[test]
+fn echo_with_complex_expression() {
+    assert_parse!("echo wibble as { this <> complex }");
+}
+
 #[test]
 fn echo_with_no_expressions_after_it() {
     assert_parse!("echo");
 }
 
+#[test]
+fn echo_with_no_expressions_after_it_but_a_message() {
+    assert_parse!("echo as message");
+}
+
 #[test]
 fn echo_with_block() {
     assert_parse!("echo { 1 + 1 }");
@@ -509,11 +529,61 @@ fn panic_with_echo() {
     assert_parse!("panic as echo \"string\"");
 }
 
+#[test]
+fn panic_with_echo_and_message() {
+    assert_parse!("panic as echo wibble as message");
+}
+
 #[test]
 fn echo_with_panic() {
     assert_parse!("echo panic as \"a\"");
 }
 
+#[test]
+fn echo_with_panic_and_message() {
+    assert_parse!("echo panic as \"a\"");
+}
+
+#[test]
+fn echo_with_panic_and_messages() {
+    assert_parse!("echo panic as \"a\" as \"b\"");
+}
+
+#[test]
+fn echo_with_assert_and_message_1() {
+    assert_parse!("assert 1 == echo 2 as this_belongs_to_echo");
+}
+
+#[test]
+fn echo_with_assert_and_message_2() {
+    assert_parse!("assert echo True as this_belongs_to_echo");
+}
+
+#[test]
+fn echo_with_assert_and_messages_1() {
+    assert_parse!("assert 1 == echo 2 as this_belongs_to_echo as this_belongs_to_assert");
+}
+
+#[test]
+fn echo_with_assert_and_messages_2() {
+    assert_parse!("assert echo True as this_belongs_to_echo as this_belongs_to_assert");
+}
+
+#[test]
+fn echo_with_assert_and_messages_3() {
+    assert_parse!("assert echo 1 == 2 as this_belongs_to_echo as this_belongs_to_assert");
+}
+
+#[test]
+fn echo_with_let_assert_and_message() {
+    assert_parse!("let assert 1 = echo 2 as this_belongs_to_echo");
+}
+
+#[test]
+fn echo_with_let_assert_and_messages() {
+    assert_parse!("let assert 1 = echo 1 as this_belongs_to_echo as this_belongs_to_assert");
+}
+
 #[test]
 fn repeated_echos() {
     assert_parse!("echo echo echo 1");
@@ -871,6 +941,32 @@ pub fn main() -> Nil {
     );
 }
 
+#[test]
+fn list_spread_followed_by_extra_item_and_another_spread() {
+    assert_module_error!(
+        r#"
+pub fn main() -> Nil {
+  let xs = [1, 2, 3]
+  let ys = [5, 6, 7]
+  [..xs, 4, ..ys]
+}
+"#
+    );
+}
+
+#[test]
+fn list_spread_followed_by_other_spread() {
+    assert_module_error!(
+        r#"
+pub fn main() -> Nil {
+  let xs = [1, 2, 3]
+  let ys = [5, 6, 7]
+  [1, ..xs, ..ys]
+}
+"#
+    );
+}
+
 #[test]
 fn list_spread_as_first_item_followed_by_other_items() {
     assert_module_error!(
@@ -1786,3 +1882,28 @@ fn assert_statement_without_expression() {
 fn assert_statement_followed_by_statement() {
     assert_error!("assert let a = 10");
 }
+
+#[test]
+fn special_error_for_pythonic_import() {
+    assert_module_error!("import gleam.io");
+}
+
+#[test]
+fn special_error_for_pythonic_neste_import() {
+    assert_module_error!("import one.two.three");
+}
+
+#[test]
+fn doesnt_issue_special_error_for_pythonic_import_if_slash() {
+    assert_module_error!("import one/two.three");
+}
+
+#[test]
+fn operator_in_pattern_size() {
+    assert_parse!("let assert <<size, payload:size(size - 1)>> = <<>>");
+}
+
+#[test]
+fn correct_precedence_in_pattern_size() {
+    assert_parse!("let assert <<size, payload:size(size + 2 * 8)>> = <<>>");
+}
