@@ -1,4 +1,4 @@
-use pubgrub::range::Range;
+use pubgrub::Range;
 
 use crate::{
     analyse::TargetSupport,
@@ -56,6 +56,10 @@ pub struct Environment<'a> {
     /// Values defined in the current function (or the prelude)
     pub scope: im::HashMap<EcoString, ValueConstructor>,
 
+    // The names of all the ignored variables and arguments in scope:
+    // `let _var = 10` `pub fn main(_var) { todo }`.
+    pub discarded_names: im::HashMap<EcoString, SrcSpan>,
+
     /// Types defined in the current module (or the prelude)
     pub module_types: HashMap<EcoString, TypeConstructor>,
 
@@ -133,6 +137,7 @@ impl<'a> Environment<'a> {
             unqualified_imported_types: HashMap::new(),
             accessors: prelude.accessors.clone(),
             scope: prelude.values.clone().into(),
+            discarded_names: im::HashMap::new(),
             importable_modules,
             current_module,
             local_variable_usages: vec![HashMap::new()],
@@ -148,6 +153,7 @@ impl<'a> Environment<'a> {
 #[derive(Debug)]
 pub struct ScopeResetData {
     local_values: im::HashMap<EcoString, ValueConstructor>,
+    discarded_names: im::HashMap<EcoString, SrcSpan>,
 }
 
 impl Environment<'_> {
@@ -170,8 +176,12 @@ impl Environment<'_> {
 
     pub fn open_new_scope(&mut self) -> ScopeResetData {
         let local_values = self.scope.clone();
+        let discarded_names = self.discarded_names.clone();
         self.local_variable_usages.push(HashMap::new());
-        ScopeResetData { local_values }
+        ScopeResetData {
+            local_values,
+            discarded_names,
+        }
     }
 
     pub fn close_scope(
@@ -180,6 +190,11 @@ impl Environment<'_> {
         was_successful: bool,
         problems: &mut Problems,
     ) {
+        let ScopeResetData {
+            local_values,
+            discarded_names,
+        } = data;
+
         let unused = self
             .local_variable_usages
             .pop()
@@ -192,7 +207,8 @@ impl Environment<'_> {
         if was_successful {
             self.handle_unused_variables(unused, problems);
         }
-        self.scope = data.local_values;
+        self.scope = local_values;
+        self.discarded_names = discarded_names;
     }
 
     pub fn next_uid(&mut self) -> u64 {
@@ -539,10 +555,10 @@ impl Environment<'_> {
                 name,
                 package,
                 module,
-                args,
+                arguments,
                 inferred_variant,
             } => {
-                let args = args
+                let arguments = arguments
                     .iter()
                     .map(|type_| self.instantiate(type_.clone(), ids, hydrator))
                     .collect();
@@ -551,7 +567,7 @@ impl Environment<'_> {
                     name: name.clone(),
                     package: package.clone(),
                     module: module.clone(),
-                    args,
+                    arguments,
                     inferred_variant: *inferred_variant,
                 })
             }
@@ -585,8 +601,11 @@ impl Environment<'_> {
                 })
             }
 
-            Type::Fn { args, return_, .. } => fn_(
-                args.iter()
+            Type::Fn {
+                arguments, return_, ..
+            } => fn_(
+                arguments
+                    .iter()
                     .map(|type_| self.instantiate(type_.clone(), ids, hydrator))
                     .collect(),
                 self.instantiate(return_.clone(), ids, hydrator),
@@ -636,9 +655,12 @@ impl Environment<'_> {
         }
     }
 
-    /// Converts entities with a usage count of 0 to warnings.
-    /// Returns the list of unused imported module location for the removed unused lsp action.
-    pub fn convert_unused_to_warnings(&mut self, problems: &mut Problems) {
+    /// Emit warnings for unused definitions, imports, expressions, etc.
+    ///
+    /// Returns the source byte start positions of all unused definitions.
+    ///
+    pub fn handle_unused(&mut self, problems: &mut Problems) -> HashSet<u32> {
+        let mut unused_positions = HashSet::new();
         let unused = self
             .local_variable_usages
             .pop()
@@ -666,18 +688,27 @@ impl Environment<'_> {
             let location = info.origin;
 
             let warning = match info.kind {
-                EntityKind::Function => Warning::UnusedPrivateFunction { location, name },
-                EntityKind::Constant => Warning::UnusedPrivateModuleConstant { location, name },
+                EntityKind::Function => {
+                    let _ = unused_positions.insert(location.start);
+                    Warning::UnusedPrivateFunction { location, name }
+                }
+                EntityKind::Constant => {
+                    let _ = unused_positions.insert(location.start);
+                    Warning::UnusedPrivateModuleConstant { location, name }
+                }
                 EntityKind::Constructor => Warning::UnusedConstructor {
                     location,
                     name,
                     imported: false,
                 },
-                EntityKind::Type => Warning::UnusedType {
-                    name,
-                    imported: false,
-                    location,
-                },
+                EntityKind::Type => {
+                    let _ = unused_positions.insert(location.start);
+                    Warning::UnusedType {
+                        name,
+                        imported: false,
+                        location,
+                    }
+                }
                 EntityKind::ImportedModule { module_name } => {
                     let _ = unused_modules.insert(module_name.clone());
                     Warning::UnusedImportedModule { name, location }
@@ -728,6 +759,8 @@ impl Environment<'_> {
             .into_iter()
             .filter(|(module, _)| !unused_modules.contains(module))
             .for_each(|(_, warning)| problems.warning(warning));
+
+        unused_positions
     }
 
     fn handle_unused_variables(
@@ -756,6 +789,10 @@ impl Environment<'_> {
             .importable_modules
             .iter()
             .filter_map(|(importable, module_info)| {
+                if module_info.is_internal && module_info.package != self.current_package {
+                    return None;
+                }
+
                 match &imported {
                     // Don't suggest importing modules if they are already imported
                     _ if self
@@ -901,17 +938,17 @@ pub fn unify(t1: Arc<Type>, t2: Arc<Type>) -> Result<(), UnifyError> {
             Type::Named {
                 module: m1,
                 name: n1,
-                args: args1,
+                arguments: arguments1,
                 ..
             },
             Type::Named {
                 module: m2,
                 name: n2,
-                args: args2,
+                arguments: arguments2,
                 ..
             },
-        ) if m1 == m2 && n1 == n2 && args1.len() == args2.len() => {
-            for (a, b) in args1.iter().zip(args2) {
+        ) if m1 == m2 && n1 == n2 && arguments1.len() == arguments2.len() => {
+            for (a, b) in arguments1.iter().zip(arguments2) {
                 unify_enclosed_type(t1.clone(), t2.clone(), unify(a.clone(), b.clone()))?;
             }
             Ok(())
@@ -935,21 +972,26 @@ pub fn unify(t1: Arc<Type>, t2: Arc<Type>) -> Result<(), UnifyError> {
 
         (
             Type::Fn {
-                args: args1,
+                arguments: arguments1,
                 return_: return1,
                 ..
             },
             Type::Fn {
-                args: args2,
+                arguments: arguments2,
                 return_: return2,
                 ..
             },
         ) => {
-            if args1.len() != args2.len() {
-                Err(unify_wrong_arity(&t1, args1.len(), &t2, args2.len()))?
+            if arguments1.len() != arguments2.len() {
+                Err(unify_wrong_arity(
+                    &t1,
+                    arguments1.len(),
+                    &t2,
+                    arguments2.len(),
+                ))?
             }
 
-            for (i, (a, b)) in args1.iter().zip(args2).enumerate() {
+            for (i, (a, b)) in arguments1.iter().zip(arguments2).enumerate() {
                 unify(a.clone(), b.clone())
                     .map_err(|_| unify_wrong_arguments(&t1, a, &t2, b, i))?;
             }
