@@ -5,7 +5,7 @@ use super::{
 use crate::{
     ast::{BinOp, BitArraySegmentTruncation, Layer, SrcSpan, TodoKind},
     build::Target,
-    type_::Type,
+    type_::{Type, expression::ComparisonOutcome},
 };
 
 use camino::Utf8PathBuf;
@@ -167,6 +167,9 @@ pub enum Error {
         location: SrcSpan,
         name: EcoString,
         variables: Vec<EcoString>,
+        /// If there's a discarded variable with the same name in the same scope
+        /// this will contain its location.
+        discarded_location: Option<SrcSpan>,
         type_with_name_in_scope: bool,
     },
 
@@ -221,6 +224,7 @@ pub enum Error {
     IncorrectArity {
         location: SrcSpan,
         expected: usize,
+        context: IncorrectArityContext,
         given: usize,
         labels: Vec<EcoString>,
     },
@@ -670,6 +674,12 @@ pub enum LiteralCollectionKind {
     Record,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum IncorrectArityContext {
+    Pattern,
+    Function,
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum InvalidImportKind {
     SrcImportingTest,
@@ -709,8 +719,15 @@ impl Named {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
-/// The origin of a variable. Used to determine how it can be ignored when unused.
-pub enum VariableOrigin {
+pub struct VariableOrigin {
+    pub syntax: VariableSyntax,
+    pub declaration: VariableDeclaration,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+/// The syntax used to define a variable. Used to determine how it can be ignored
+/// when unused.
+pub enum VariableSyntax {
     /// A variable that can be ignored by prefixing with an underscore, `_name`
     Variable(EcoString),
     /// A variable from label shorthand syntax, which can be ignored with an underscore: `label: _`
@@ -721,17 +738,34 @@ pub enum VariableOrigin {
     Generated,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+/// The source of a variable, such as a `let` assignment, or function parameter.
+pub enum VariableDeclaration {
+    LetPattern,
+    UsePattern,
+    ClausePattern,
+    FunctionParameter,
+    Generated,
+}
+
 impl VariableOrigin {
     pub fn how_to_ignore(&self) -> Option<String> {
-        match self {
-            VariableOrigin::Variable(name) => {
+        match &self.syntax {
+            VariableSyntax::Variable(name) => {
                 Some(format!("You can ignore it with an underscore: `_{name}`."))
             }
-            VariableOrigin::LabelShorthand(label) => Some(format!(
+            VariableSyntax::LabelShorthand(label) => Some(format!(
                 "You can ignore it with an underscore: `{label}: _`."
             )),
-            VariableOrigin::AssignmentPattern => Some("You can safely remove it.".to_string()),
-            VariableOrigin::Generated => None,
+            VariableSyntax::AssignmentPattern => Some("You can safely remove it.".to_string()),
+            VariableSyntax::Generated => None,
+        }
+    }
+
+    pub fn generated() -> Self {
+        Self {
+            syntax: VariableSyntax::Generated,
+            declaration: VariableDeclaration::Generated,
         }
     }
 }
@@ -922,8 +956,8 @@ pub enum Warning {
     TodoOrPanicUsedAsFunction {
         kind: TodoOrPanic,
         location: SrcSpan,
-        args_location: Option<SrcSpan>,
-        args: usize,
+        arguments_location: Option<SrcSpan>,
+        arguments: usize,
     },
 
     UnreachableCodeAfterPanic {
@@ -967,12 +1001,12 @@ pub enum Warning {
         location: SrcSpan,
     },
 
-    /// When we are trying to use bool assert on a literal value. For example:
+    /// When we are trying to use bool assert on a literal boolean. For example:
     /// ```gleam
     /// assert True
     ///        ^ The programmer knows this will never panic, so it's useless
     /// ```
-    AssertLiteralValue {
+    AssertLiteralBool {
         location: SrcSpan,
     },
 
@@ -991,6 +1025,27 @@ pub enum Warning {
         first: SrcSpan,
         second: SrcSpan,
     },
+
+    /// Top-level definition should not shadow an imported one.
+    /// This includes constant or function imports.
+    TopLevelDefinitionShadowsImport {
+        location: SrcSpan,
+        name: EcoString,
+    },
+
+    /// This warning is raised when we perform a comparison that the compiler
+    /// can tell is always going to succeed or fail. For example:
+    ///
+    /// ```gleam
+    /// 1 == 1 // This always succeeds
+    /// 2 != 2 // This always fails
+    /// 1 > 10 // This always fails
+    /// a == a // This always succeeds
+    /// ```
+    RedundantComparison {
+        location: SrcSpan,
+        outcome: ComparisonOutcome,
+    },
 }
 
 #[derive(Debug, Eq, Copy, PartialEq, Clone, serde::Serialize, serde::Deserialize)]
@@ -1214,11 +1269,13 @@ impl Warning {
             | Warning::RedundantPipeFunctionCapture { location, .. }
             | Warning::FeatureRequiresHigherGleamVersion { location, .. }
             | Warning::JavaScriptIntUnsafe { location, .. }
-            | Warning::AssertLiteralValue { location, .. }
+            | Warning::AssertLiteralBool { location, .. }
             | Warning::BitArraySegmentTruncatedValue { location, .. }
+            | Warning::TopLevelDefinitionShadowsImport { location, .. }
             | Warning::ModuleImportedTwice {
                 second: location, ..
-            } => *location,
+            }
+            | Warning::RedundantComparison { location, .. } => *location,
         }
     }
 
@@ -1265,6 +1322,7 @@ pub fn convert_get_value_constructor_error(
             location,
             name,
             variables,
+            discarded_location: None,
             type_with_name_in_scope,
         },
 
@@ -1372,7 +1430,7 @@ pub enum MatchFunTypeError {
     IncorrectArity {
         expected: usize,
         given: usize,
-        args: Vec<Arc<Type>>,
+        arguments: Vec<Arc<Type>>,
         return_type: Arc<Type>,
     },
     NotFn {
@@ -1395,6 +1453,7 @@ pub fn convert_not_fun_error(
         ) => Error::IncorrectArity {
             labels: vec![],
             location: call_location,
+            context: IncorrectArityContext::Function,
             expected,
             given,
         },
