@@ -117,13 +117,13 @@ impl Hydrator {
                 name_location,
                 module,
                 name,
-                arguments: args,
+                arguments,
             }) => {
                 // Hydrate the type argument AST into types
-                let mut argument_types = Vec::with_capacity(args.len());
-                for t in args {
-                    let type_ = self.type_from_ast(t, environment, problems)?;
-                    argument_types.push((t.location(), type_));
+                let mut argument_types = Vec::with_capacity(arguments.len());
+                for argument in arguments {
+                    let type_ = self.type_from_ast(argument, environment, problems)?;
+                    argument_types.push((argument.location(), type_));
                 }
 
                 // Look up the constructor
@@ -183,12 +183,12 @@ impl Hydrator {
                 }
 
                 // Ensure that the correct number of arguments have been given to the constructor
-                if args.len() != parameters.len() {
+                if arguments.len() != parameters.len() {
                     return Err(Error::IncorrectTypeArity {
                         location: *location,
                         name: name.clone(),
                         expected: parameters.len(),
-                        given: args.len(),
+                        given: arguments.len(),
                     });
                 }
 
@@ -221,16 +221,14 @@ impl Hydrator {
             )),
 
             TypeAst::Fn(TypeAstFn {
-                arguments: args,
-                return_,
-                ..
+                arguments, return_, ..
             }) => {
-                let args = args
+                let arguments = arguments
                     .iter()
                     .map(|type_| self.type_from_ast(type_, environment, problems))
                     .try_collect()?;
                 let return_ = self.type_from_ast(return_, environment, problems)?;
-                Ok(fn_(args, return_))
+                Ok(fn_(arguments, return_))
             }
 
             TypeAst::Var(TypeAstVar { name, location }) => {
