@@ -1,4 +1,5 @@
-use hexpm::version::Version;
+use ecow::eco_format;
+use hexpm::version::{LowestVersion, Version};
 use im::hashmap;
 use itertools::Itertools;
 use num_bigint::BigInt;
@@ -10,8 +11,10 @@ use super::*;
 use crate::{
     analyse::{self, Inferred, name::check_name_case},
     ast::{
-        AssignName, BitArrayOption, ImplicitCallArgOrigin, Layer, UntypedPatternBitArraySegment,
+        AssignName, BitArrayOption, BitArraySize, ImplicitCallArgOrigin, Layer, TypedBitArraySize,
+        UntypedPatternBitArraySegment,
     },
+    parse::PatternPosition,
     reference::ReferenceKind,
     type_::expression::FunctionDefinition,
 };
@@ -45,6 +48,9 @@ pub struct PatternTyper<'a, 'b> {
     /// keeps track of whether it is in scope, so that we can correctly detect
     /// valid/invalid uses.
     variables: HashMap<EcoString, LocalVariable>,
+
+    /// What kind of pattern we are typing
+    position: PatternPosition,
 }
 
 #[derive(Debug)]
@@ -96,6 +102,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         current_function: &'a FunctionDefinition,
         hydrator: &'a Hydrator,
         problems: &'a mut Problems,
+        position: PatternPosition,
     ) -> Self {
         Self {
             environment,
@@ -109,6 +116,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
             problems,
             error_encountered: false,
             variables: HashMap::new(),
+            position,
         }
     }
 
@@ -209,11 +217,20 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 let _ = self
                     .inferred_variant_variables
                     .insert(name.clone(), variant_index);
+
+                let origin = match &variable.variant {
+                    ValueConstructorVariant::LocalVariable { origin, .. } => origin.clone(),
+                    ValueConstructorVariant::ModuleConstant { .. }
+                    | ValueConstructorVariant::LocalConstant { .. }
+                    | ValueConstructorVariant::ModuleFn { .. }
+                    | ValueConstructorVariant::Record { .. } => VariableOrigin::generated(),
+                };
+
                 // This variable is only inferred in this branch of the case expression
                 self.environment.insert_local_variable(
                     name.clone(),
                     variable.definition_location().span,
-                    VariableOrigin::Variable(name),
+                    origin,
                     type_,
                 );
             }
@@ -444,7 +461,11 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
             .try_collect()
             .expect("The function always returns Ok");
 
-        let segment_type = match bit_array::type_options_for_pattern(&options, !is_last_segment) {
+        let segment_type = match bit_array::type_options_for_pattern(
+            &options,
+            !is_last_segment,
+            self.environment.target,
+        ) {
             Ok(type_) => type_,
             Err(error) => {
                 self.error(Error::BitArraySegmentError {
@@ -473,11 +494,11 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     // Int segments that aren't a whole number of bytes
                     BitArrayOption::<TypedPattern>::Size { value, .. } if segment_type.is_int() => {
                         match &**value {
-                            Pattern::<_>::Int {
+                            Pattern::BitArraySize(BitArraySize::Int {
                                 location,
                                 int_value,
                                 ..
-                            } if int_value % 8 != BigInt::ZERO => {
+                            }) if int_value % 8 != BigInt::ZERO => {
                                 self.track_feature_usage(
                                     FeatureKind::JavaScriptUnalignedBitArray,
                                     *location,
@@ -571,6 +592,10 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         match pattern {
             Pattern::Discard { name, location, .. } => {
                 self.check_name_case(location, &name, Named::Discard);
+                let _ = self
+                    .environment
+                    .discarded_names
+                    .insert(name.clone(), location);
                 Pattern::Discard {
                     type_,
                     name,
@@ -596,50 +621,14 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 }
             }
 
-            Pattern::VarUsage { name, location, .. } => {
-                let constructor = match self.variables.get_mut(&name) {
-                    // If we've bound a variable in the current bit array pattern, we
-                    // want to use that.
-                    Some(variable) if variable.in_scope() => {
-                        variable.usage = Usage::UsedInPattern;
-                        ValueConstructor::local_variable(
-                            variable.location,
-                            variable.origin.clone(),
-                            variable.type_.clone(),
-                        )
+            Pattern::BitArraySize(size) => {
+                let location = size.location();
+                match self.bit_array_size(size, type_.clone()) {
+                    Ok(size) => Pattern::BitArraySize(size),
+                    Err(error) => {
+                        self.error(error);
+                        Pattern::Invalid { location, type_ }
                     }
-                    // Otherwise, we check the local scope.
-                    Some(_) | None => match self.environment.get_variable(&name) {
-                        Some(constructor) => constructor.clone(),
-                        None => {
-                            self.error(Error::UnknownVariable {
-                                location,
-                                name: name.clone(),
-                                variables: self.environment.local_value_names(),
-                                type_with_name_in_scope: self
-                                    .environment
-                                    .module_types
-                                    .keys()
-                                    .any(|type_| type_ == &name),
-                            });
-                            return Pattern::Invalid { location, type_ };
-                        }
-                    },
-                };
-
-                self.environment.increment_usage(&name);
-                let type_ = self.environment.instantiate(
-                    constructor.type_.clone(),
-                    &mut hashmap![],
-                    self.hydrator,
-                );
-                self.unify_types(int(), type_.clone(), location);
-
-                Pattern::VarUsage {
-                    name,
-                    location,
-                    constructor: Some(constructor),
-                    type_,
                 }
             }
 
@@ -660,7 +649,10 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                         left,
                         string(),
                         *left_location,
-                        VariableOrigin::AssignmentPattern,
+                        VariableOrigin {
+                            syntax: VariableSyntax::AssignmentPattern,
+                            declaration: self.position.to_declaration(),
+                        },
                     );
                 }
 
@@ -671,13 +663,18 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             right,
                             string(),
                             right_location,
-                            VariableOrigin::Variable(right.clone()),
+                            VariableOrigin {
+                                syntax: VariableSyntax::Variable(right.clone()),
+                                declaration: self.position.to_declaration(),
+                            },
                         );
                     }
-                    AssignName::Discard(_) => {
-                        if let AssignName::Discard(right) = &right_side_assignment {
-                            self.check_name_case(right_location, right, Named::Discard);
-                        }
+                    AssignName::Discard(right) => {
+                        let _ = self
+                            .environment
+                            .discarded_names
+                            .insert(right.clone(), right_location);
+                        self.check_name_case(right_location, right, Named::Discard);
                     }
                 };
 
@@ -701,7 +698,10 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     &name,
                     pattern.type_().clone(),
                     location,
-                    VariableOrigin::AssignmentPattern,
+                    VariableOrigin {
+                        syntax: VariableSyntax::AssignmentPattern,
+                        declaration: self.position.to_declaration(),
+                    },
                 );
                 Pattern::Assign {
                     name,
@@ -752,7 +752,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 elements,
                 tail,
                 ..
-            } => match type_.get_app_args(
+            } => match type_.get_app_arguments(
                 Publicity::Public,
                 PRELUDE_PACKAGE_NAME,
                 PRELUDE_MODULE_NAME,
@@ -760,8 +760,8 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 1,
                 self.environment,
             ) {
-                Some(args) => {
-                    let type_ = args
+                Some(arguments) => {
+                    let type_ = arguments
                         .first()
                         .expect("Failed to get type argument of List")
                         .clone();
@@ -802,6 +802,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                         self.error(Error::IncorrectArity {
                             labels: vec![],
                             location,
+                            context: IncorrectArityContext::Pattern,
                             expected: type_elements.len(),
                             given: elements.len(),
                         });
@@ -854,7 +855,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                 module,
                 name_location,
                 name,
-                arguments: mut pattern_args,
+                arguments: mut pattern_arguments,
                 spread,
                 ..
             } => {
@@ -880,7 +881,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             location,
                             name_location,
                             name,
-                            arguments: self.infer_pattern_call_args(pattern_args, &[]),
+                            arguments: self.infer_pattern_call_arguments(pattern_arguments, &[]),
                             module,
                             constructor: Inferred::Unknown,
                             spread,
@@ -889,13 +890,14 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     }
                 };
 
+                let mut incorrect_arity_error = false;
                 match constructor.field_map() {
                     // The fun has a field map so labelled arguments may be present and need to be reordered.
                     Some(field_map) => {
                         if let Some(spread_location) = spread {
                             // Using the spread operator when you have already provided variables for all of the
                             // record's fields throws an error
-                            if pattern_args.len() == field_map.arity as usize {
+                            if pattern_arguments.len() == field_map.arity as usize {
                                 {
                                     self.problems.error(Error::UnnecessarySpreadOperator {
                                         location: spread_location,
@@ -911,10 +913,10 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             // we have calculate that index and then insert() the discards. It would be faster
                             // if we could put the discards anywhere which would let us use push().
                             // Potential future optimisation.
-                            let index_of_first_labelled_arg = pattern_args
+                            let index_of_first_labelled_arg = pattern_arguments
                                 .iter()
                                 .position(|argument| argument.label.is_some())
-                                .unwrap_or(pattern_args.len());
+                                .unwrap_or(pattern_arguments.len());
 
                             // In Gleam we can pass in positional unlabelled args to a constructor
                             // even if the field was defined as labelled
@@ -938,16 +940,16 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             //            │     ╰ We supplied 1 labelled arg
                             //            ╰ We supplied 2 unlabelled args
                             //
-                            let supplied_unlabelled_args = index_of_first_labelled_arg;
-                            let supplied_labelled_args = pattern_args
+                            let supplied_unlabelled_arguments = index_of_first_labelled_arg;
+                            let supplied_labelled_arguments = pattern_arguments
                                 .iter()
                                 .filter_map(|argument| argument.label.clone())
                                 .collect::<HashSet<_>>();
-                            let constructor_unlabelled_args =
+                            let constructor_unlabelled_arguments =
                                 field_map.arity - field_map.fields.len() as u32;
                             let labelled_arguments_supplied_as_unlabelled =
-                                supplied_unlabelled_args
-                                    .saturating_sub(constructor_unlabelled_args as usize);
+                                supplied_unlabelled_arguments
+                                    .saturating_sub(constructor_unlabelled_arguments as usize);
 
                             let mut missing_labels = field_map
                                 .fields
@@ -960,9 +962,9 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                                 .skip(labelled_arguments_supplied_as_unlabelled)
                                 // ... lastly we still need to remove all those labels that
                                 // were explicitly supplied in the pattern.
-                                .filter(|label| !supplied_labelled_args.contains(label));
+                                .filter(|label| !supplied_labelled_arguments.contains(label));
 
-                            while pattern_args.len() < field_map.arity as usize {
+                            while pattern_arguments.len() < field_map.arity as usize {
                                 let new_call_arg = CallArg {
                                     value: Pattern::Discard {
                                         name: "_".into(),
@@ -974,21 +976,24 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                                     implicit: Some(ImplicitCallArgOrigin::PatternFieldSpread),
                                 };
 
-                                pattern_args.insert(index_of_first_labelled_arg, new_call_arg);
+                                pattern_arguments.insert(index_of_first_labelled_arg, new_call_arg);
                             }
                         }
 
-                        if let Err(error) = field_map.reorder(&mut pattern_args, location) {
-                            {
-                                self.problems.error(error);
-                                self.error_encountered = true;
-                            };
+                        if let Err(error) = field_map.reorder(
+                            &mut pattern_arguments,
+                            location,
+                            IncorrectArityContext::Pattern,
+                        ) {
+                            incorrect_arity_error = true;
+                            self.problems.error(error);
+                            self.error_encountered = true;
                         }
                     }
 
                     None => {
                         // The fun has no field map and so we error if arguments have been labelled
-                        match assert_no_labelled_arguments(&pattern_args) {
+                        match assert_no_labelled_arguments(&pattern_arguments) {
                             Ok(()) => {}
                             Err(error) => {
                                 self.problems.error(error);
@@ -1000,8 +1005,8 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             if let ValueConstructorVariant::Record { arity, .. } =
                                 &constructor.variant
                             {
-                                while pattern_args.len() < usize::from(*arity) {
-                                    pattern_args.push(CallArg {
+                                while pattern_arguments.len() < usize::from(*arity) {
+                                    pattern_arguments.push(CallArg {
                                         value: Pattern::Discard {
                                             name: "_".into(),
                                             location: spread_location,
@@ -1082,7 +1087,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                     self.environment
                         .instantiate(constructor_type, &mut hashmap![], self.hydrator);
                 match instantiated_constructor_type.deref() {
-                    Type::Fn { args, return_ } => {
+                    Type::Fn { arguments, return_ } => {
                         self.unify_types(type_.clone(), return_.clone(), location);
 
                         if let Some((variable_to_infer, inferred_variant)) =
@@ -1091,16 +1096,21 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             self.set_subject_variable_variant(variable_to_infer, inferred_variant);
                         }
 
-                        if args.len() != pattern_args.len() {
+                        // We're emitting the incorrect arity error only if we haven't emitted
+                        // one already. This might happen when we can't reorder the field map
+                        // of a constructor because there's not enough labels.
+                        if arguments.len() != pattern_arguments.len() && !incorrect_arity_error {
                             self.error(Error::IncorrectArity {
                                 labels: vec![],
                                 location,
-                                expected: args.len(),
-                                given: pattern_args.len(),
+                                context: IncorrectArityContext::Pattern,
+                                expected: arguments.len(),
+                                given: pattern_arguments.len(),
                             });
                         }
 
-                        let pattern_args = self.infer_pattern_call_args(pattern_args, args);
+                        let pattern_arguments =
+                            self.infer_pattern_call_arguments(pattern_arguments, arguments);
 
                         Pattern::Constructor {
                             location,
@@ -1108,7 +1118,7 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             name,
                             module,
                             constructor: Inferred::Known(pattern_constructor),
-                            arguments: pattern_args,
+                            arguments: pattern_arguments,
                             spread,
                             type_: return_.clone(),
                         }
@@ -1125,12 +1135,13 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
                             self.set_subject_variable_variant(variable_to_infer, inferred_variant);
                         }
 
-                        if !pattern_args.is_empty() {
+                        if !pattern_arguments.is_empty() {
                             self.error(Error::IncorrectArity {
                                 labels: vec![],
                                 location,
+                                context: IncorrectArityContext::Pattern,
                                 expected: 0,
-                                given: pattern_args.len(),
+                                given: pattern_arguments.len(),
                             });
                         }
                         Pattern::Constructor {
@@ -1151,12 +1162,12 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
         }
     }
 
-    fn infer_pattern_call_args(
+    fn infer_pattern_call_arguments(
         &mut self,
-        pattern_args: Vec<CallArg<UntypedPattern>>,
+        pattern_arguments: Vec<CallArg<UntypedPattern>>,
         expected_types: &[Arc<Type>],
     ) -> Vec<CallArg<TypedPattern>> {
-        pattern_args
+        pattern_arguments
             .into_iter()
             .enumerate()
             .map(|(index, arg)| {
@@ -1187,6 +1198,101 @@ impl<'a, 'b> PatternTyper<'a, 'b> {
             .collect()
     }
 
+    fn bit_array_size(
+        &mut self,
+        size: BitArraySize<()>,
+        type_: Arc<Type>,
+    ) -> Result<TypedBitArraySize, Error> {
+        let typed_size = match size {
+            BitArraySize::Int {
+                location,
+                value,
+                int_value,
+            } => {
+                self.unify_types(type_, int(), location);
+
+                if self.environment.target == Target::JavaScript
+                    && !self.current_function.has_javascript_external
+                {
+                    check_javascript_int_safety(&int_value, location, self.problems);
+                }
+
+                BitArraySize::Int {
+                    location,
+                    value,
+                    int_value,
+                }
+            }
+            BitArraySize::Variable { name, location, .. } => {
+                let constructor = match self.variables.get_mut(&name) {
+                    // If we've bound a variable in the current bit array pattern,
+                    // we want to use that.
+                    Some(variable) if variable.in_scope() => {
+                        variable.usage = Usage::UsedInPattern;
+                        ValueConstructor::local_variable(
+                            variable.location,
+                            variable.origin.clone(),
+                            variable.type_.clone(),
+                        )
+                    }
+                    // Otherwise, we check the local scope.
+                    Some(_) | None => match self.environment.get_variable(&name) {
+                        Some(constructor) => constructor.clone(),
+                        None => {
+                            return Err(Error::UnknownVariable {
+                                location,
+                                name: name.clone(),
+                                variables: self.environment.local_value_names(),
+                                discarded_location: self
+                                    .environment
+                                    .discarded_names
+                                    .get(&eco_format!("_{name}"))
+                                    .cloned(),
+                                type_with_name_in_scope: self
+                                    .environment
+                                    .module_types
+                                    .keys()
+                                    .any(|type_| type_ == &name),
+                            });
+                        }
+                    },
+                };
+
+                self.environment.increment_usage(&name);
+                let type_ = self.environment.instantiate(
+                    constructor.type_.clone(),
+                    &mut hashmap![],
+                    self.hydrator,
+                );
+                self.unify_types(int(), type_.clone(), location);
+
+                BitArraySize::Variable {
+                    name,
+                    location,
+                    constructor: Some(Box::new(constructor)),
+                    type_,
+                }
+            }
+            BitArraySize::BinaryOperator {
+                location,
+                operator,
+                left,
+                right,
+            } => BitArraySize::BinaryOperator {
+                location,
+                operator,
+                left: Box::new(self.bit_array_size(*left, type_.clone())?),
+                right: Box::new(self.bit_array_size(*right, type_)?),
+            },
+            BitArraySize::Block { location, inner } => BitArraySize::Block {
+                location,
+                inner: Box::new(self.bit_array_size(*inner, type_)?),
+            },
+        };
+
+        Ok(typed_size)
+    }
+
     fn check_name_case(&mut self, location: SrcSpan, name: &EcoString, kind: Named) {
         if let Err(error) = check_name_case(location, name, kind) {
             self.problems.error(error);
