@@ -1,5 +1,6 @@
 use crate::{
-    assert_error, assert_internal_module_error, assert_module_error, assert_module_syntax_error,
+    assert_error, assert_internal_module_error, assert_js_module_error, assert_module_error,
+    assert_module_syntax_error,
 };
 
 #[test]
@@ -2871,6 +2872,21 @@ fn echo_followed_by_no_expression() {
     assert_error!("echo");
 }
 
+#[test]
+fn echo_followed_by_no_expression_and_message() {
+    assert_error!("echo as \"wibble\"");
+}
+
+#[test]
+fn echo_followed_by_no_expression_and_invalid_message() {
+    assert_error!("echo as 1");
+}
+
+#[test]
+fn echo_followed_by_invalid_message() {
+    assert_error!("echo 11 as { True || False }");
+}
+
 #[test]
 fn echo_followed_by_no_expression_2() {
     assert_module_error!(
@@ -3125,3 +3141,145 @@ fn bit_array_using_pattern_variables_from_other_bit_array() {
 fn non_utf8_string_assignment() {
     assert_error!(r#"let assert <<"Hello" as message:utf16>> = <<>>"#);
 }
+
+#[test]
+fn shadowed_function_argument() {
+    assert_module_error!(
+        "
+pub fn go(_x) {
+  x + 1
+}
+"
+    );
+}
+
+#[test]
+fn shadowed_fn_argument() {
+    assert_module_error!(
+        "
+pub fn go(x) {
+  fn(_y) {
+    y + x
+  }
+}
+"
+    );
+}
+
+#[test]
+fn shadowed_let_variable() {
+    assert_module_error!(
+        "
+pub fn go() {
+  let _x = 1
+  x + 1
+}
+"
+    );
+}
+
+#[test]
+fn shadowed_pattern_variable() {
+    assert_module_error!(
+        "
+pub type Wibble {
+  Wibble(Int)
+}
+
+pub fn go(x) {
+  case x {
+    Wibble(_y) -> y + 1
+  }
+}
+"
+    );
+}
+
+#[test]
+fn do_not_suggest_ignored_variable_outside_of_current_scope() {
+    assert_module_error!(
+        "
+pub fn go() {
+  let _ = {
+    let _y = 1 // <- this shouldn't be highlighted!
+  }
+  y
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4693
+#[test]
+fn pattern_with_incorrect_arity() {
+    assert_module_error!(
+        "
+pub type Pokemon { Pokemon(name: String, id: Int) }
+
+pub fn main() {
+  case todo {
+    Pokemon(name:) -> todo
+  }
+}
+"
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/3884
+#[test]
+fn show_only_missing_labels() {
+    assert_module_error!(
+        "
+fn wibble(a a: Int, b b: Float, c c: String) {
+    todo
+}
+
+pub fn wobble() {
+    wibble(1, 2.0)
+}
+"
+    );
+}
+#[test]
+fn native_endianness_javascript_target() {
+    assert_js_module_error!(
+        "
+pub fn main() {
+  let assert <<a:native>> = <<10>>
+}
+"
+    );
+}
+
+#[test]
+fn utf8_codepoint_javascript_target() {
+    assert_js_module_error!(
+        "
+pub fn main() {
+  let assert <<a:utf8_codepoint>> = <<10>>
+}
+"
+    );
+}
+
+#[test]
+fn utf16_codepoint_javascript_target() {
+    assert_js_module_error!(
+        "
+pub fn main() {
+  let assert <<a:utf16_codepoint>> = <<10>>
+}
+"
+    );
+}
+
+#[test]
+fn utf32_codepoint_javascript_target() {
+    assert_js_module_error!(
+        "
+pub fn main() {
+  let assert <<a:utf32_codepoint>> = <<10>>
+}
+"
+    );
+}
