@@ -9,6 +9,7 @@ use crate::{
             FeatureKind, LiteralCollectionKind, PanicPosition, TodoOrPanic,
             UnreachablePatternReason,
         },
+        expression::ComparisonOutcome,
         pretty::Printer,
     },
 };
@@ -998,14 +999,14 @@ Either change the pattern or use `panic` to unconditionally fail.",
                 type_::Warning::TodoOrPanicUsedAsFunction {
                     kind,
                     location,
-                    args_location,
-                    args,
+                    arguments_location,
+                    arguments,
                 } => {
                     let title = match kind {
                         TodoOrPanic::Todo => "Todo used as a function".into(),
                         TodoOrPanic::Panic => "Panic used as a function".into(),
                     };
-                    let label_location = match args_location {
+                    let label_location = match arguments_location {
                         None => location,
                         Some(location) => location,
                     };
@@ -1014,7 +1015,7 @@ Either change the pattern or use `panic` to unconditionally fail.",
                         TodoOrPanic::Panic => "panic",
                     };
                     let mut text = format!("`{name}` is not a function");
-                    match args {
+                    match arguments {
                         0 => text.push_str(&format!(
                             ", you can just write `{name}` instead of `{name}()`."
                         )),
@@ -1026,7 +1027,7 @@ Either change the pattern or use `panic` to unconditionally fail.",
                         ),
                     };
 
-                    match args {
+                    match arguments {
                         0 => {}
                         _ => text.push_str(&format!(
                             "\n\nHint: if you want to display an error message you should write
@@ -1258,11 +1259,11 @@ doesn't fit in that many {unit}. It would be truncated by taking its {taken}, re
                     }
                 }
 
-                type_::Warning::AssertLiteralValue { location } => Diagnostic {
+                type_::Warning::AssertLiteralBool { location } => Diagnostic {
                     title: "Assertion of a literal value".into(),
                     text: wrap(
-                        "Asserting that a literal value is redundant since you \
-can already tell whether it will be true or false.",
+                        "Asserting on a literal bool is redundant since you \
+can already tell whether it will be `True` or `False`.",
                     ),
                     hint: None,
                     level: diagnostic::Level::Warning,
@@ -1270,7 +1271,7 @@ can already tell whether it will be true or false.",
                         src: src.clone(),
                         path: path.to_path_buf(),
                         label: diagnostic::Label {
-                            text: Some("This is always the same".into()),
+                            text: None,
                             span: *location,
                         },
                         extra_labels: Vec::new(),
@@ -1290,18 +1291,68 @@ can already tell whether it will be true or false.",
                         src: src.clone(),
                         path: path.to_path_buf(),
                         label: diagnostic::Label {
-                            text: None,
-                            span: *first,
+                            text: Some("Reimported here".into()),
+                            span: *second,
                         },
                         extra_labels: vec![ExtraLabel {
                             src_info: None,
                             label: diagnostic::Label {
-                                text: None,
-                                span: *second,
+                                text: Some("First imported here".into()),
+                                span: *first,
                             },
                         }],
                     }),
                 },
+
+                type_::Warning::TopLevelDefinitionShadowsImport { location, name } => {
+                    let text = format!(
+                        "Definition of {name} shadows an imported value.
+The imported value could not be used in this module anyway."
+                    );
+                    Diagnostic {
+                        title: "Shadowed Import".into(),
+                        text: wrap(&text),
+                        level: diagnostic::Level::Warning,
+                        location: Some(Location {
+                            path: path.clone(),
+                            src: src.clone(),
+                            label: diagnostic::Label {
+                                text: Some(wrap(&format!("`{name}` is defined here"))),
+                                span: *location,
+                            },
+                            extra_labels: Vec::new(),
+                        }),
+                        hint: Some("Either rename the definition or remove the import.".into()),
+                    }
+                }
+
+                type_::Warning::RedundantComparison { location, outcome } => Diagnostic {
+                    title: "Redundant comparison".into(),
+                    text: format!(
+                        "This comparison is redundant since it always {}.",
+                        match outcome {
+                            ComparisonOutcome::AlwaysSucceeds => "succeeds",
+                            ComparisonOutcome::AlwaysFails => "fails",
+                        }
+                    ),
+                    hint: None,
+                    level: diagnostic::Level::Warning,
+                    location: Some(Location {
+                        label: diagnostic::Label {
+                            text: Some(format!(
+                                "This is always `{}`",
+                                match outcome {
+                                    ComparisonOutcome::AlwaysSucceeds => "True",
+                                    ComparisonOutcome::AlwaysFails => "False",
+                                }
+                            )),
+                            span: *location,
+                        },
+                        path: path.clone(),
+                        src: src.clone(),
+                        extra_labels: vec![],
+                    }),
+                },
             },
 
             Warning::DeprecatedEnvironmentVariable { variable } => {
